'''
Created on 06-09-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''

import os
import re
import wx
import wx.animate
import wx.lib.scrolledpanel as scrolled
import wx.lib.delayedresult

from aqopa.model import name_indexes
from aqopa.bin import gui as aqopa_gui
from aqopa.simulator.error import RuntimeException

class SingleVersionPanel(wx.Panel):
    """ 
    Frame presenting results for one simulation.  
    Simulator may be retrived from module, 
    because each module has its own simulator.
    """
    
    def __init__(self, module, *args, **kwargs):
        wx.Panel.__init__(self, *args, **kwargs)

        self.module = module
        self.versionSimulator = {}

        self.hostChoosePanels       = []  # Panels used to choose hosts for times results
        self.checkBoxInformations   = []  # Tuples with host name, and index ranges widget
        self.hostCheckBoxes         = []  # List of checkboxes with hosts names used for hosts' selection

        self.timeResultsPanel       = None

        #################
        # VERSION BOX
        #################
        
        versionBox = wx.StaticBox(self, label="Version")
        self.versionsList = wx.ComboBox(self, style=wx.TE_READONLY)
        self.versionsList.Bind(wx.EVT_COMBOBOX, self.OnVersionChanged)
        
        versionBoxSizer = wx.StaticBoxSizer(versionBox, wx.VERTICAL)
        versionBoxSizer.Add(self.versionsList, 1, wx.ALL | wx.ALIGN_CENTER, 5)

        #################
        # TOTAL TIME BOX
        #################
        
        self.totalTimeBox = wx.StaticBox(self, label="Total time")
        self.totalTimeLabel = wx.StaticText(self, label="---")
        
        totalTimeBoxSizer = wx.StaticBoxSizer(self.totalTimeBox, wx.VERTICAL)
        totalTimeBoxSizer.Add(self.totalTimeLabel, 1, wx.ALL | wx.ALIGN_CENTER, 5)
        
        #################
        # TIMES BOX
        #################
        
        self.timesBox = wx.StaticBox(self, label="Times")

        operationBox, operationBoxSizer = self._BuildOperationsBoxAndSizer()
        hostsBox, hostsBoxSizer = self._BuildHostsBoxAndSizer()

        timesBoxSizer = wx.StaticBoxSizer(self.timesBox, wx.VERTICAL)
        
        timesHBoxSizer = wx.BoxSizer(wx.HORIZONTAL)
        timesHBoxSizer.Add(operationBoxSizer, 0, wx.ALL | wx.EXPAND)
        timesHBoxSizer.Add(hostsBoxSizer, 1, wx.ALL | wx.EXPAND)
        
        self.showTimeBtn = wx.Button(self, label="Show time")
        self.showTimeBtn.Bind(wx.EVT_BUTTON, self.OnShowTimeButtonClicked)
        
        self.timesResultBox = wx.StaticBox(self, label="Results")
        self.timesResultBoxSizer = wx.StaticBoxSizer(self.timesResultBox, wx.VERTICAL)
        
        timesBoxSizer.Add(timesHBoxSizer, 0, wx.ALL | wx.EXPAND)
        timesBoxSizer.Add(self.showTimeBtn, 0, wx.ALL | wx.EXPAND)
        timesBoxSizer.Add(self.timesResultBoxSizer, 1, wx.ALL | wx.EXPAND)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(versionBoxSizer, 0, wx.ALL | wx.EXPAND, 5)
        sizer.Add(totalTimeBoxSizer, 0, wx.ALL | wx.EXPAND, 5)
        sizer.Add(timesBoxSizer, 1, wx.ALL | wx.EXPAND, 5)
        self.SetSizer(sizer)
        
        self.SetVersionsResultsVisibility(False)
        
    #################
    # REACTIONS
    #################
        
    def AddFinishedSimulation(self, simulator):
        """ """
        version = simulator.context.version
        self.versionsList.Append(version.name)
        
        self.versionSimulator[version.name] = simulator
        
    def OnVersionChanged(self, event):
        """ """
        versionName = self.versionsList.GetValue()
        simulator = self.versionSimulator[versionName]
        
        totalTime = self.GetTotalTime(simulator)
        self.totalTimeLabel.SetLabel("%.2f ms" % totalTime)
        
        self._BuildHostsChoosePanel(simulator)
        
        self.SetVersionsResultsVisibility(True)
        
    def OnShowTimeButtonClicked(self, event):
        """ """
        versionName = self.versionsList.GetValue()
        simulator = self.versionSimulator[versionName]
        hosts = self._GetSelectedHosts(simulator)
        
        if len(hosts) == 0:
            wx.MessageBox("Please select hosts.", 'Error', wx.OK | wx.ICON_ERROR)
            return
        
        if self.oneTimeRB.GetValue():
            self.ShowHostsTimes(simulator, hosts)
        elif self.avgTimeRB.GetValue():
            self.ShowAverageHostsTime(simulator, hosts)
        elif self.minTimeRB.GetValue():
            self.ShowMinimalHostsTime(simulator, hosts)
        elif self.maxTimeRB.GetValue():
            self.ShowMaximalHostsTime(simulator, hosts)
            
    def RemoveAllSimulations(self):
        """ """
        self.versionsList.Clear()
        self.versionsList.SetValue("")
        self.versionSimulator = {}

        self.hostChoosePanels = []
        self.checkBoxInformations = {}
        self.hostCheckBoxes = []
        self.hostsBoxSizer.Clear(True)

        self.SetVersionsResultsVisibility(False)

    #################
    # LAYOUT
    #################
    
    def _BuildOperationsBoxAndSizer(self):
        """ """
        self.operationBox = wx.StaticBox(self, label="Operation")
        
        self.oneTimeRB = wx.RadioButton(self, label="Total host's time")
        self.avgTimeRB = wx.RadioButton(self, label="Average hosts' time")
        self.minTimeRB = wx.RadioButton(self, label="Minimal hosts' time")
        self.maxTimeRB = wx.RadioButton(self, label="Maximal hosts' time")
        
        operationBoxSizer = wx.StaticBoxSizer(self.operationBox, wx.VERTICAL)
        operationBoxSizer.Add(self.oneTimeRB, 0, wx.ALL)
        operationBoxSizer.Add(self.avgTimeRB, 0, wx.ALL)
        operationBoxSizer.Add(self.minTimeRB, 0, wx.ALL)
        operationBoxSizer.Add(self.maxTimeRB, 0, wx.ALL)
        
        return self.operationBox, operationBoxSizer
    
    def _BuildHostsBoxAndSizer(self):
        """ """
        self.hostsBox = wx.StaticBox(self, label="Host(s)")
        self.hostsBoxSizer = wx.StaticBoxSizer(self.hostsBox, wx.VERTICAL)
        return self.hostsBox, self.hostsBoxSizer
    
    def _BuildHostsChoosePanel(self, simulator):
        """ """
        for p in self.hostChoosePanels:
            p.Destroy()
        self.hostChoosePanels = []
        self.checkBoxInformations = {}
        self.hostCheckBoxes = []
        
        self.hostsBoxSizer.Layout()
        
        hosts = simulator.context.hosts
        hostsIndexes = {} 
        for h in hosts:
            name = h.original_name()
            indexes = name_indexes(h.name)
            index = indexes[0]
            
            if name not in hostsIndexes or index > hostsIndexes[name]:
                hostsIndexes[name] = index
                
        for hostName in hostsIndexes:
            
            panel = wx.Panel(self)
            panelSizer = wx.BoxSizer(wx.HORIZONTAL)
            
            ch = wx.CheckBox(panel, label=hostName, size=(120, 20))
            textCtrl = wx.TextCtrl(panel)
            textCtrl.SetValue("0")
            
            rangeLabel = "Available range: 0"
            if hostsIndexes[hostName] > 0:
                rangeLabel += " - %d" % hostsIndexes[hostName] 
            maxLbl = wx.StaticText(panel, label=rangeLabel)
            
            panelSizer.Add(ch, 0, wx.ALL | wx.ALIGN_CENTER)
            panelSizer.Add(textCtrl, 0, wx.ALL | wx.ALIGN_CENTER)
            panelSizer.Add(maxLbl, 0, wx.ALL | wx.ALIGN_CENTER)
            panel.SetSizer(panelSizer)
            self.hostsBoxSizer.Add(panel, 1, wx.ALL)
            
            self.checkBoxInformations[ch] = (hostName, textCtrl)
            self.hostChoosePanels.append(panel)
            self.hostCheckBoxes.append(ch)
            
        self.hostsBoxSizer.Layout()
        self.Layout()

    def SetVersionsResultsVisibility(self, visible):
        """ """
        widgets = []
        widgets.append(self.timesBox)
        widgets.append(self.totalTimeBox)
        widgets.append(self.totalTimeLabel)
        widgets.append(self.operationBox)
        widgets.append(self.oneTimeRB)
        widgets.append(self.avgTimeRB)
        widgets.append(self.minTimeRB)
        widgets.append(self.maxTimeRB)
        widgets.append(self.hostsBox)
        widgets.append(self.showTimeBtn)
        widgets.append(self.timesResultBox)
        
        for w in widgets:
            if visible:
                w.Show()
            else:
                w.Hide()
                
        self.Layout()
    
    #################
    # STATISTICS
    #################
        
    def _GetSelectedHosts(self, simulator):
        """ Returns list of hosts selected by user """
        
        def ValidateHostsRange(indexesRange):
            """ """
            return re.match(r'\d(-\d)?(,\d(-\d)?)*', indexesRange)
        
        def GetIndexesFromRange(indexesRange):
            """ Extracts numbers list of hosts from range text """
            indexes = []
            ranges = indexesRange.split(',')
            for r in ranges:
                parts = r.split('-')
                if len(parts) == 1:
                    indexes.append(int(parts[0]))
                else:
                    for i in range(int(parts[0]), int(parts[1])+1):
                        indexes.append(i)
            return indexes
        
        hosts = []
        for ch in self.hostCheckBoxes:
            if not ch.IsChecked():
                continue
            hostName, hostRangeTextCtrl = self.checkBoxInformations[ch]
            indexesRange = hostRangeTextCtrl.GetValue()
            if not ValidateHostsRange(indexesRange):
                wx.MessageBox("Range '%s' for host '%s' is invalid. Valid example: 0,12,20-25,30." 
                              % (indexesRange, hostName), 'Error', wx.OK | wx.ICON_ERROR)
                break
            else:
                indexes = GetIndexesFromRange(indexesRange)
                for h in simulator.context.hosts:
                    hostIndexes = name_indexes(h.name)
                    if h.original_name() == hostName and hostIndexes[0] in indexes:
                        hosts.append(h)
        return hosts
        
    def GetTotalTime(self, simulator):
        """ Return total time of simulated version. """
        totalTime = 0
        hosts = simulator.context.hosts
        for h in hosts:
            t = self.module.get_current_time(simulator, h)
            if t > totalTime:
                totalTime = t
        return totalTime 
    
    def ShowHostsTimes(self, simulator, hosts):
        """ """
        if self.timeResultsPanel:
            self.timeResultsPanel.Destroy()
            self.timeResultsPanel = None
            
        self.timeResultsPanel = scrolled.ScrolledPanel(self)
        self.timesResultBoxSizer.Add(self.timeResultsPanel, 1, wx.ALL | wx.EXPAND, 5)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        for h in hosts:
            
            lblText = "%s: %.2f ms" % (h.name,self.module.get_current_time(simulator, h))
            
            error = h.get_finish_error()
            if error is not None:
                lblText += " (Not Finished - %s)" % error
            
            lbl = wx.StaticText(self.timeResultsPanel, label=lblText)
            sizer.Add(lbl)
        
        self.timeResultsPanel.SetSizer(sizer)
        self.timeResultsPanel.SetupScrolling(scroll_x=False)
        self.Layout()
    
    def ShowAverageHostsTime(self, simulator, hosts):
        """ """
        def GetVal(simulator, hosts):
            sum = 0.0
            n = len(hosts)
            for h in hosts:
                sum += self.module.get_current_time(simulator, h)
            return sum / float(n)
        
        if self.timeResultsPanel:
            self.timeResultsPanel.Destroy()
            
        self.timeResultsPanel = scrolled.ScrolledPanel(self)
        self.timesResultBoxSizer.Add(self.timeResultsPanel, 1, wx.ALL | wx.EXPAND, 5)
    
        avg = GetVal(simulator, hosts)
        lblText = "Average: %.2f ms" % avg
        lbl = wx.StaticText(self.timeResultsPanel, label=lblText)        
    
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(lbl)
        
        self.timeResultsPanel.SetSizer(sizer)
        self.Layout()
        
    def ShowMinimalHostsTime(self, simulator, hosts):
        """ """
        def GetVal(simulator, hosts):
            val = None 
            for h in hosts:
                t = self.module.get_current_time(simulator, h)
                if val is None or t < val:
                    val = t
            return val
        
        if self.timeResultsPanel:
            self.timeResultsPanel.Destroy()
            
        self.timeResultsPanel = scrolled.ScrolledPanel(self)
        self.timesResultBoxSizer.Add(self.timeResultsPanel, 1, wx.ALL | wx.EXPAND, 5)
    
        val = GetVal(simulator, hosts)
        lblText = "Minimum: %.2f ms" % val
        lbl = wx.StaticText(self.timeResultsPanel, label=lblText)        
    
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(lbl)
        
        self.timeResultsPanel.SetSizer(sizer)
        self.Layout()
    
    def ShowMaximalHostsTime(self, simulator, hosts):
        """ """
        def GetVal(simulator, hosts):
            val = 0.0 
            for h in hosts:
                t = self.module.get_current_time(simulator, h)
                if t > val:
                    val = t
            return val
        
        if self.timeResultsPanel:
            self.timeResultsPanel.Destroy()
            
        self.timeResultsPanel = scrolled.ScrolledPanel(self)
        self.timesResultBoxSizer.Add(self.timeResultsPanel, 1, wx.ALL | wx.EXPAND, 5)
    
        val = GetVal(simulator, hosts)
        lblText = "Maximum: %.2f ms" % val
        lbl = wx.StaticText(self.timeResultsPanel, label=lblText)     
    
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(lbl)
        
        self.timeResultsPanel.SetSizer(sizer)
        self.Layout()
        
#TIME_TYPE_MAX   = 1
#TIME_TYPE_AVG   = 2
#TIME_TYPE_TOTAL = 3
#        
#class VersionsChartsPanel(wx.Panel):
#    
#    def __init__(self, module, *args, **kwargs):
#        wx.Panel.__init__(self, *args, **kwargs)
#
#        self.module = module
#
#        self.simulators = []
#        self.chartPanel = None
#        
#        sizer = wx.BoxSizer(wx.VERTICAL)
#        
#        self.chartTypeBox = wx.StaticBox(self, label="Chart Type")
#        self.chartTypeBoxSizer = wx.StaticBoxSizer(self.chartTypeBox, wx.HORIZONTAL)
#        
#        self.totalTimeOnRepetitionBtn = wx.Button(self, label="T_Total / N")
#        self.avgTimeOnRepetitionBtn = wx.Button(self, label="T_AVG / N")
#        self.totalTimeOnVersionsBtn = wx.Button(self, label="T_Total / Version")
#        self.totalTimeOnMetricsBtn = wx.Button(self, label="T_Total / M")
#        self.avgTimeOnMetricsBtn = wx.Button(self, label="T_AVG / M")
#        
#        self.totalTimeOnRepetitionBtn.Bind(wx.EVT_BUTTON, self.OnTimeTotalOnRepetitionBtnClicked)
#        self.avgTimeOnRepetitionBtn.Bind(wx.EVT_BUTTON, self.OnTimeAvgOnRepetitionBtnClicked)
#        self.totalTimeOnVersionsBtn.Bind(wx.EVT_BUTTON, self.OnTimeTotalOnVersionsBtnClicked)
#        self.totalTimeOnMetricsBtn.Bind(wx.EVT_BUTTON, self.OnTimeTotalOnMetricsBtnClicked)
#        self.avgTimeOnMetricsBtn.Bind(wx.EVT_BUTTON, self.OnTimeAvgOnMetricsBtnClicked)
#        
#        self.chartTypeBoxSizer.Add(self.totalTimeOnRepetitionBtn, 1, wx.ALL | wx.ALIGN_CENTER, 5)
#        self.chartTypeBoxSizer.Add(self.avgTimeOnRepetitionBtn, 1, wx.ALL | wx.ALIGN_CENTER, 5)
#        self.chartTypeBoxSizer.Add(self.totalTimeOnVersionsBtn, 1, wx.ALL | wx.ALIGN_CENTER, 5)
#        self.chartTypeBoxSizer.Add(self.totalTimeOnMetricsBtn, 1, wx.ALL | wx.ALIGN_CENTER, 5)
#        self.chartTypeBoxSizer.Add(self.avgTimeOnMetricsBtn, 1, wx.ALL | wx.ALIGN_CENTER, 5)
#        
#        sizer.Add(self.chartTypeBoxSizer, 0, wx.ALL | wx.EXPAND, 5)
#        
#        
#        self.chartConfigBox = wx.StaticBox(self, label="Chart Configuration")
#        self.chartConfigBoxSizer = wx.StaticBoxSizer(self.chartConfigBox, wx.VERTICAL)
#        
#        sizer.Add(self.chartConfigBoxSizer, 1, wx.ALL | wx.EXPAND, 5)
#        
#        self.SetSizer(sizer)
#        
#    def _GetHostsRepetitions(self):
#        hostsRepetitions = {}
#        
#        for s in self.simulators:
#            for rh in s.context.version.run_hosts:
#                if rh.host_name not in hostsRepetitions:
#                    hostsRepetitions[rh.host_name] = 1
#                if rh.repetitions > hostsRepetitions[rh.host_name]:
#                    hostsRepetitions[rh.host_name] = rh.repetitions
#        
#        return hostsRepetitions
#    
#    def _GetHosts(self, simulators, hostName):
#        simulatorsHosts = {}
#        for s in simulators:
#            simulatorsHosts[s] = []
#            for h in s.context.hosts:
#                if h.original_name() == hostName:
#                    simulatorsHosts[s].append(h)
#        return simulatorsHosts
#    
#    def _MetricExistsIn(self, metric, metrics):
#        """ """
#        for m in metrics:
#            if self._AreMetricsEqual(m, metric):
#                return True
#        return False
#    
#    def _AreMetricsEqual(self, leftMetric, rightMetric):
#        """ """
#        if len(leftMetric) != len(rightMetric):
#            return False
#        
#        for leftMetricsSet in leftMetric:
#            found = False
#            for rightMetricsSet in rightMetric:
#                if leftMetricsSet.host_name == rightMetricsSet.host_name:
#                    found= True
#                    if leftMetricsSet.configuration_name != rightMetricsSet.configuration_name:
#                        return False
#            if not found:
#                return False
#        return True
#    
#    def _GetMetric(self, simulator):
#        """ """
#        return simulator.context.version.metrics_sets
#    
#    def _GetMetrics(self):
#        """ """
#        metrics = []
#        for s in self.simulators:
#            metric = self._GetMetric(s)
#            if not self._MetricExistsIn(metric, metrics):
#                metrics.append(metric)
#        return metrics
#        
#    #################
#    # REACTIONS
#    #################
#    
#    def OnTimeTotalOnRepetitionBtnClicked(self, event):
#        self.chartConfigBox.SetLabel("T_Total/N Chart Configuration")
#        self.BuildTimesByRepetitionsChartPanel(TIME_TYPE_MAX)
#    
#    def OnTimeAvgOnRepetitionBtnClicked(self, event):
#        self.chartConfigBox.SetLabel("TAvg/N Chart Configuration")
#        self.BuildTimesByRepetitionsChartPanel(TIME_TYPE_AVG)
#    
#    def OnTimeTotalOnVersionsBtnClicked(self, event):
#        self.chartConfigBox.SetLabel("TTotal/Version Chart Configuration")
#        self.BuildTimesByVersionsChartPanel(TIME_TYPE_TOTAL)
#    
#    def OnTimeTotalOnMetricsBtnClicked(self, event):
#        self.chartConfigBox.SetLabel("T_Total/M Chart Configuration")
#        self.BuildTimesByMetricsChartPanel(TIME_TYPE_MAX)
#    
#    def OnTimeAvgOnMetricsBtnClicked(self, event):
#        self.chartConfigBox.SetLabel("TAvg/M Chart Configuration")
#        self.BuildTimesByMetricsChartPanel(TIME_TYPE_AVG)
#        
#    def OnShowChartTTotalVarRepButtonClicked(self, event):
#        """ """
#        if len(self.curves) == 0:
#            wx.MessageBox("No curves defined. You must add at least one curve.", 
#                          'Error', wx.OK | wx.ICON_ERROR)
#        else:
#            self.CalculateAndShowChartByRepFrame(TIME_TYPE_MAX)
#    
#    def OnShowChartTAvgVarRepButtonClicked(self, event):
#        """ """
#        if len(self.curves) == 0:
#            wx.MessageBox("No curves defined. You must add at least one curve.", 
#                          'Error', wx.OK | wx.ICON_ERROR)
#        else:
#            self.CalculateAndShowChartByRepFrame(TIME_TYPE_AVG)
#        
#    def OnShowChartTTotalVarMetricsButtonClicked(self, event):
#        """ """
#        if len(self.curves) == 0:
#            wx.MessageBox("No curves defined. You must add at least one curve.", 
#                          'Error', wx.OK | wx.ICON_ERROR)
#        else:
#            self.CalculateAndShowChartByMetricsFrame(TIME_TYPE_MAX)
#    
#    def OnShowChartTAvgVarMetricsButtonClicked(self, event):
#        """ """
#        if len(self.curves) == 0:
#            wx.MessageBox("No curves defined. You must add at least one curve.", 
#                          'Error', wx.OK | wx.ICON_ERROR)
#        else:
#            self.CalculateAndShowChartByMetricsFrame(TIME_TYPE_AVG)
#        
#    def OnShowChartTTotalVarVersionButtonClicked(self, event):
#        """ """
#        if len(self.simulators) == 0:
#            wx.MessageBox("There are no finished simulations yet. Please wait and try again.", 
#                          'Error', wx.OK | wx.ICON_ERROR)
#        else:
#            self.CalculateAndShowChartByVersions()
#        
#    def OnAddCurveButtonClicked(self, event):
#        """ """
#        lblParts = []
#        curveSimulators = []
#        for ch in self.checkboxSimulator:
#            if ch.IsChecked():
#                curveSimulators.append(self.checkboxSimulator[ch])
#                lblParts.append(self.checkboxSimulator[ch].context.version.name)
#        
#        if len(curveSimulators) == 0:
#            return
#        
#        self.curves.append(curveSimulators)
#        
#        curveLabel = "%d. Versions: %s" % (len(self.curves), ', '.join(lblParts))
#        text = wx.StaticText(self.curvesListPanel, label=curveLabel)
#        self.curvesListPanelSizer.Add(text)
#        
#        self.Layout()
#    
#    #################
#    # LAYOUT
#    #################
#    
#    def BuildCurvesPanel(self, parent, parentSizer, onShowChartClicked):
#        """ """
#        rightBox = wx.StaticBox(parent, label="Define curves")
#        rightBoxSizer = wx.StaticBoxSizer(rightBox, wx.HORIZONTAL)
#        parentSizer.Add(rightBoxSizer, 4, wx.ALL, 5)
#        
#        addCurveBox = wx.StaticBox(parent, label="Add curve")
#        addCurveBoxSizer = wx.StaticBoxSizer(addCurveBox, wx.VERTICAL)
#        rightBoxSizer.Add(addCurveBoxSizer, 0, wx.ALL, 5)
#        
#        self.checkboxSimulator = {} 
#        
#        for s in self.simulators:
#            version = s.context.version
#            ch = wx.CheckBox(parent, label=version.name)
#            addCurveBoxSizer.Add(ch)
#            self.checkboxSimulator[ch] = s
#            
#        addCurveButton = wx.Button(parent, label="Add curve")
#        addCurveBoxSizer.Add(addCurveButton, 0, wx.ALIGN_CENTER)
#        addCurveButton.Bind(wx.EVT_BUTTON, self.OnAddCurveButtonClicked)
#        
#        curvesBox = wx.StaticBox(parent, label="Curves")
#        curvesBoxSizer = wx.StaticBoxSizer(curvesBox, wx.VERTICAL)
#        rightBoxSizer.Add(curvesBoxSizer, 1, wx.ALL, 5)
#        
#        self.curvesListPanel = wx.Panel(parent)
#        self.curvesListPanelSizer = wx.BoxSizer(wx.VERTICAL)
#        self.curvesListPanel.SetSizer(self.curvesListPanelSizer)
#        curvesBoxSizer.Add(self.curvesListPanel, 1, wx.ALL | wx.EXPAND, 5)
#        
#        showChartBtn = wx.Button(parent, label="Show Chart")
#        showChartBtn.Bind(wx.EVT_BUTTON, onShowChartClicked)
#        curvesBoxSizer.Add(showChartBtn, 0, wx.ALIGN_CENTER)
#    
#    def BuildTimesByRepetitionsChartPanel(self, timeType):
#        """ """
#        if self.chartPanel:
#            self.chartPanel.Destroy()
#            self.chartPanel = None
#            
#        self.chartPanel = wx.Panel(self)
#        chartPanelSizer = wx.BoxSizer(wx.HORIZONTAL)
#        self.chartPanel.SetSizer(chartPanelSizer)
#        
#        leftBox = wx.StaticBox(self.chartPanel, label="Choose host")
#        leftBoxSizer = wx.StaticBoxSizer(leftBox, wx.VERTICAL)
#        chartPanelSizer.Add(leftBoxSizer, 1, wx.ALL, 5)
#        
#        self.hostRadios = []
#        
#        hostsRepetitions = self._GetHostsRepetitions()
#        for hostName in hostsRepetitions:
#            radioBtn = wx.RadioButton(self.chartPanel, label=hostName)
#            leftBoxSizer.Add(radioBtn)
#            self.hostRadios.append(radioBtn)
#        
#        onShowChartBtnClicked = None
#        if timeType == TIME_TYPE_MAX:
#            onShowChartBtnClicked = self.OnShowChartTTotalVarRepButtonClicked
#        elif timeType == TIME_TYPE_AVG:
#            onShowChartBtnClicked = self.OnShowChartTAvgVarRepButtonClicked
#        
#        self.BuildCurvesPanel(self.chartPanel, chartPanelSizer, onShowChartBtnClicked)
#        
#        self.chartConfigBoxSizer.Add(self.chartPanel, 1, wx.ALL | wx.EXPAND, 5)
#        self.Layout()
#        
#        self.curves = []
#    
#    def BuildTimesByVersionsChartPanel(self, timeType):
#        """ """
#        if self.chartPanel:
#            self.chartPanel.Destroy()
#            self.chartPanel = None
#            
#        self.chartPanel = wx.Panel(self)
#        chartPanelSizer = wx.BoxSizer(wx.HORIZONTAL)
#        self.chartPanel.SetSizer(chartPanelSizer)
#        
#        self.BuildCurvesPanel(self.chartPanel, chartPanelSizer, self.OnShowChartTTotalVarVersionButtonClicked)
#        
#        self.chartConfigBoxSizer.Add(self.chartPanel, 1, wx.ALL | wx.EXPAND, 5)
#        self.Layout()
#        
#        self.curves = []
#    
#    def BuildTimesByMetricsChartPanel(self, timeType):
#        """ """
#        if self.chartPanel:
#            self.chartPanel.Destroy()
#            self.chartPanel = None
#            
#        self.chartPanel = wx.Panel(self)
#        chartPanelSizer = wx.BoxSizer(wx.HORIZONTAL)
#        self.chartPanel.SetSizer(chartPanelSizer)
#        
#        leftBox = wx.StaticBox(self.chartPanel, label="Metrics")
#        leftBoxSizer = wx.StaticBoxSizer(leftBox, wx.VERTICAL)
#        chartPanelSizer.Add(leftBoxSizer, 0, wx.ALL, 5)
#        
#        self.metrics = self._GetMetrics()
#        
#        i = 0
#        for metric in self.metrics:
#            i += 1
#            metricPanel = wx.Panel(self.chartPanel)
#            metricPanelSizer = wx.BoxSizer(wx.HORIZONTAL)
#            metricPanel.SetSizer(metricPanelSizer)
#            
#            leftBoxSizer.Add(metricPanel)
#            
#            metricNumber = "%d ." % i
#            lbl = wx.StaticText(metricPanel, label=metricNumber)
#            metricPanelSizer.Add(lbl)
#            
#            metricsConfigPanel = wx.Panel(metricPanel)
#            metricsConfigPanelSizer = wx.BoxSizer(wx.VERTICAL)
#            metricsConfigPanel.SetSizer(metricsConfigPanelSizer)
#            
#            metricPanelSizer.Add(metricsConfigPanel)
#            
#            for mSet in metric:
#                lbl = wx.StaticText(metricsConfigPanel, label="Host: %s -> Config: %s" % (mSet.host_name, mSet.configuration_name))
#                metricsConfigPanelSizer.Add(lbl)
#        
#        onShowChartBtnClicked = None
#        if timeType == TIME_TYPE_MAX:
#            onShowChartBtnClicked = self.OnShowChartTTotalVarMetricsButtonClicked
#        elif timeType == TIME_TYPE_AVG:
#            onShowChartBtnClicked = self.OnShowChartTAvgVarMetricsButtonClicked
#        
#        self.BuildCurvesPanel(self.chartPanel, chartPanelSizer, onShowChartBtnClicked)
#        
#        self.chartConfigBoxSizer.Add(self.chartPanel, 1, wx.ALL | wx.EXPAND, 5)
#        self.Layout()
#        
#        self.curves = []
#        
#    def CalculateAndShowChartByRepFrame(self, timeType):
#        """ """
#        
#        def TTotal(module, simulator, hosts):
#            val = 0.0 
#            for h in hosts:
#                t = module.get_current_time(simulator, h)
#                if t > val:
#                    val = t
#            return val
#        
#        def TAvg(module, simulator, hosts):
#            s = 0.0 
#            for h in hosts:
#                s += module.get_current_time(simulator, h)
#            l = len(hosts)
#            if l == 0:
#                return  0
#            return s / float(l)
#        
#        hostName = None
#        for radio in self.hostRadios:
#            if radio.GetValue():
#                hostName = radio.GetLabel()
#                break
#        if not hostName:
#            return
#        
#        curvesData = []
#        
#        chartFun = lambda x : x
#        chartTitle  = ""
#        xLabel      = ""
#        yLabel      = ""
#        
#        if timeType == TIME_TYPE_MAX:
#            chartFun    = TTotal
#            chartTitle  = "Chart: Total Time / Repetitions"
#            xLabel      = "Repetitions"
#            yLabel      = "T_Total"
#        else:
#            chartFun    = TAvg
#            chartTitle  = "Chart: TimeAvg / Repetitions"
#            xLabel      = "Repetitions"
#            yLabel      = "TAvg"
#        
#        i = 0
#        for curveSimulators in self.curves:
#            
#            i += 1
#            label = "%d." % i
#            
#            values = []
#            hostsBySimulator = self._GetHosts(curveSimulators, hostName)
#            for s in hostsBySimulator:
#                values.append((len(hostsBySimulator[s]), chartFun(self.module, s, hostsBySimulator[s])))
#
#            values.sort(key=lambda t: t[0])                
#            curveData = (label, values)
#            curvesData.append(curveData)
#            
#        self.ShowChartFrame(chartTitle, xLabel, yLabel, curvesData)
#        
#    def CalculateAndShowChartByMetricsFrame(self, timeType):
#        """ """
#        
#        def TTotal(values):
#            val = 0.0 
#            for t in values:
#                if t > val:
#                    val = t
#            return val
#        
#        def TAvg(values):
#            s = 0.0 
#            for v in values:
#                s += v
#            l = len(values)
#            if l == 0:
#                return  0
#            return s / float(l)
#        
#        def buildLegend(parent):
#            
#            mainBox = wx.StaticBox(parent, label="Metrics")
#            mainBoxSizer = wx.StaticBoxSizer(mainBox,wx.VERTICAL)
#            
#            i = 0
#            for metric in self.metrics:
#                i += 1
#                metricPanel = wx.Panel(parent)
#                metricPanelSizer = wx.BoxSizer(wx.HORIZONTAL)
#                metricPanel.SetSizer(metricPanelSizer)
#                
#                mainBoxSizer.Add(metricPanel)
#                
#                metricNumber = "%d ." % i
#                lbl = wx.StaticText(metricPanel, label=metricNumber)
#                metricPanelSizer.Add(lbl)
#                
#                metricsConfigPanel = wx.Panel(metricPanel)
#                metricsConfigPanelSizer = wx.BoxSizer(wx.VERTICAL)
#                metricsConfigPanel.SetSizer(metricsConfigPanelSizer)
#                
#                metricPanelSizer.Add(metricsConfigPanel)
#
#                for mSet in metric:
#                    lbl = wx.StaticText(metricsConfigPanel, label="Host: %s -> Config: %s" % (mSet.host_name, mSet.configuration_name))
#                    metricsConfigPanelSizer.Add(lbl)
#        
#            return mainBoxSizer
#        
#        curvesData = []
#        
#        chartFun = lambda x : x
#        chartTitle  = ""
#        xLabel      = ""
#        yLabel      = ""
#        
#        if timeType == TIME_TYPE_MAX:
#            chartFun    = TTotal
#            chartTitle  = "Chart: Total Time / Metric"
#            xLabel      = "Metric"
#            yLabel      = "T_Total"
#        else:
#            chartFun    = TAvg
#            chartTitle  = "Chart: TimeAvg / Metric"
#            xLabel      = "Metric"
#            yLabel      = "TAvg"
#        
#        c = 0
#        for curveSimulators in self.curves:
#            c += 1
#
#            values = []
#            i = 0
#            for m in self.metrics:
#                i += 1
#                time_values = []
#                
#                for s in curveSimulators:
#                    currentMetric = self._GetMetric(s)
#                    if self._AreMetricsEqual(m, currentMetric):
#                        time_values.extend([ self.module.get_current_time(s, h) 
#                                            for h in s.context.hosts])
#                
#                values.append((i, chartFun(time_values)))
#            
#            values.sort(key=lambda t: t[0])                
#            curveData = ("%d." % c, values)
#            curvesData.append(curveData)
#            
#        self.ShowChartFrame(chartTitle, xLabel, yLabel, curvesData, 
#                            buildLegendPanelFun=buildLegend)
#        
#    def CalculateAndShowChartByVersions(self):
#        """ """
#        def chartFun(module, simulator, hosts):
#            val = 0.0 
#            for h in hosts:
#                t = module.get_current_time(simulator, h)
#                if t > val:
#                    val = t
#            return val
#        
#        chartTitle  = "TTotal / Version"
#        xLabel      = "Version"
#        yLabel      = "TTotal"
#        curvesData  = []
#
#        c = 0
#        for curveSimulators in self.curves:
#            c += 1
#
#            values = []
#            i = 0
#            for s in curveSimulators:
#                i += 1
#                values.append((i, chartFun(self.module, s, s.context.hosts)))
#                
#            values.sort(key=lambda t: t[0]) 
#
#            curveData = ("%d." % c, values)
#            curvesData.append(curveData)
#            
#        self.ShowChartFrame(chartTitle, xLabel, yLabel, curvesData)
#        
#    def AddFinishedSimulation(self, simulator):
#        """ """
#        self.simulators.append(simulator)
#
#        self.chartTypeBox.Show()
#        self.totalTimeOnRepetitionBtn.Show()
#        self.totalTimeOnMetricsBtn.Show()
#        self.totalTimeOnVersionsBtn.Show()
#        self.avgTimeOnRepetitionBtn.Show()
#        self.avgTimeOnMetricsBtn.Show()
#
#        self.chartConfigBox.Show()
#        self.chartConfigBox.SetLabel("Chart Configuration")
#        self.Layout()
#
#    def RemoveAllSimulations(self):
#        """ """
#        self.simulators = []
#        self.chartTypeBox.Hide()
#        self.totalTimeOnRepetitionBtn.Hide()
#        self.totalTimeOnMetricsBtn.Hide()
#        self.totalTimeOnVersionsBtn.Hide()
#        self.avgTimeOnRepetitionBtn.Hide()
#        self.avgTimeOnMetricsBtn.Hide()
#
#        self.chartConfigBox.Hide()
#        self.chartConfigBoxSizer.Clear(True)
#        self.Layout()
#        
#    def ShowChartFrame(self, chartTitle, xTitle, yTitle, curvesData, 
#                       buildLegendPanelFun = None):
#        """ Shows frame with gnuplot chart """
#        
#        from wx.lib.plot import PlotCanvas, PolyMarker, PolyLine, PlotGraphics
#        import random 
#        
#        def drawPlot(chartTitle, xTitle, yTitle, curvesData):
#            """ """
#            plots = []
#            
#            for curveData in curvesData:
#                cr = random.randint(0, 255)
#                cg = random.randint(0, 255)
#                cb = random.randint(0, 255)
#                markers = PolyMarker(curveData[1], legend=curveData[0], 
#                                     colour=wx.Color(cr,cg,cb), size=2)
#                line = PolyLine(curveData[1], legend=curveData[0], 
#                                colour=wx.Color(cr,cg,cb), width=1)
#                plots.append(markers)
#                plots.append(line)
#            
#            return PlotGraphics(plots, chartTitle, 
#                                xTitle, yTitle)
#        
#        frame = wx.Frame(None, title=chartTitle)
#        panel = wx.Panel(frame)
#        panelSizer = wx.BoxSizer(wx.HORIZONTAL)
#        panel.SetSizer(panelSizer)
#        
#        frameSizer = wx.BoxSizer(wx.VERTICAL)
#        frameSizer.Add(panel, 1, wx.ALL | wx.EXPAND, 5)
#        frame.SetSizer(frameSizer)
#        
#        canvas = PlotCanvas(panel)
#        canvas.Draw(drawPlot(chartTitle, xTitle, yTitle, curvesData))
#        
#        if buildLegendPanelFun:
#            legendPanel = buildLegendPanelFun(panel)
#            panelSizer.Add(legendPanel, 0, wx.ALL, 5)
#        panelSizer.Add(canvas, 1, wx.EXPAND)
#        
#        panelSizer.Layout()
#        
#        frame.Maximize(True)
#        frame.Show()
    
TIME_TYPE_TOTAL = 1
TIME_TYPE_AVG   = 2
    
class DistributedSystemOptimizationPanel(wx.ScrolledWindow):
    
    def __init__(self, module, *args, **kwargs):
        wx.ScrolledWindow.__init__(self, *args, **kwargs)

        self.module = module
        self.checkBoxes = []
        self.checkBoxToSimulator = {}
        self.optimizationResults = {}
        
        # OPTIMIZATION CONFIGURATION
        
        descText = wx.StaticText(self, label="Optimization algorithm " + \
        "finds the numbers of simultaneous clients for each version " + \
        "such that the execution time of protocols will be the same (with given tolerance)."
        , style=wx.TE_MULTILINE | wx.TE_READONLY)
        
        configurationBox = wx.StaticBox(self, label="Optimization configuration")
        configurationBoxSizer = wx.StaticBoxSizer(configurationBox, wx.VERTICAL)
        
        hostSizer = wx.BoxSizer(wx.HORIZONTAL)
        hostText = wx.StaticText(self, label="Repeated host:", size=(200, -1))
        self.hostCombo = wx.ComboBox(self, style=wx.TE_READONLY, size=(200, -1))
        hostSizer.Add(hostText, 1)
        hostSizer.Add(self.hostCombo, 1)
        
        versionsSizer = wx.BoxSizer(wx.HORIZONTAL)
        versionsText = wx.StaticText(self, label="Versions:", size=(200, -1))
        self.versionsSelectSizer = wx.BoxSizer(wx.VERTICAL)  
        versionsSizer.Add(versionsText, 1)  
        versionsSizer.Add(self.versionsSelectSizer, 1)
        
        typeSizer = wx.BoxSizer(wx.HORIZONTAL)
        timeTypeText = wx.StaticText(self, label="Time type:", size=(200, -1))
        timeTypeSelectSizer = wx.BoxSizer(wx.VERTICAL)
        self.avgRadioBtn = wx.RadioButton(self, label="Average")
        self.avgRadioBtn.SetValue(True)
        self.totalRadioBtn = wx.RadioButton(self, label="Total")
        timeTypeSelectSizer.Add(self.avgRadioBtn)
        timeTypeSelectSizer.Add(self.totalRadioBtn)
        typeSizer.Add(timeTypeText, 1)
        typeSizer.Add(timeTypeSelectSizer, 1)
        
        toleranceSizer = wx.BoxSizer(wx.HORIZONTAL)
        toleranceText = wx.StaticText(self, label="Tolerance: (in %)", size=(200, -1))
        self.toleranceTextCtrl = wx.TextCtrl(self)
        toleranceSizer.Add(toleranceText, 1)
        toleranceSizer.Add(self.toleranceTextCtrl, 1)
        
        self.startButton = wx.Button(self, label="Start optimization")
        self.startButton.Bind(wx.EVT_BUTTON, self.OnStartClick)
        
        configurationBoxSizer.Add(hostSizer, 0, wx.ALIGN_CENTER|wx.ALL, 10)
        configurationBoxSizer.Add(versionsSizer, 0, wx.ALIGN_CENTER|wx.ALL, 10)
        configurationBoxSizer.Add(typeSizer, 0, wx.ALIGN_CENTER|wx.ALL, 10)
        configurationBoxSizer.Add(toleranceSizer, 0, wx.ALIGN_CENTER|wx.ALL, 10)
        configurationBoxSizer.Add(self.startButton, 0, wx.ALIGN_CENTER|wx.ALL, 10)
        
        # OPTIMIZATION PROCESS
        self.module.get_gui().Bind(aqopa_gui.EVT_MODULE_SIMULATION_ALLOWED, self.OnSimulationAllowed)
        
        self.interpreter = None # Interpreter used to run simulators
        
        self.maxTime = 0 # Maximum execution time (avg or total)
        self.timeType = None # The type of time calculation (avg or total)
        self.hostName = None # Name of host used to calculate avg time
        self.maxSimulator = None # The simulator with maximum execution time
        self.maxRepetition = 0 # The number of simulatenous clients in scenario with maximum execution time 
        
        self.newSimulator = None
        self.previousRepetition = 0
        self.previousTime = 0
        self.currentRepetition = 0
        self.currentTime = 0
        self.tolerance = 0.05
        
        self.progressTimer      = wx.Timer(self)
        self.dots = 0
        self.Bind(wx.EVT_TIMER, self.OnProgressTimerTick, self.progressTimer)
        
        self.processBox = wx.StaticBox(self, label="Optimization process")
        processBoxSizer = wx.StaticBoxSizer(self.processBox, wx.VERTICAL)
        
        self.statusText = wx.StaticText(self, label="Not started")
        self.dotsText = wx.StaticText(self, label="")
        self.dotsText.Hide()
        self.repetitionText = wx.StaticText(self, label="")
        self.repetitionText.Hide()
        
        self.maximumBox = wx.StaticBox(self, label="Version with maximum execution time")
        maximumBoxSizer = wx.StaticBoxSizer(self.maximumBox, wx.VERTICAL)
        
        hs = wx.BoxSizer(wx.HORIZONTAL)
        self.versionsLabel = wx.StaticText(self, label="Version:")
        hs.Add(self.versionsLabel, 0)
        self.maximumVersionText = wx.StaticText(self, label="")
        hs.Add(self.maximumVersionText, 0, wx.ALIGN_RIGHT)
        maximumBoxSizer.Add(hs, 0)
        
        hs = wx.BoxSizer(wx.HORIZONTAL)
        self.timeLabel = wx.StaticText(self, label="Time:")
        hs.Add(self.timeLabel, 0)
        self.maximumTimeText = wx.StaticText(self, label="")
        hs.Add(self.maximumTimeText, 0, wx.ALIGN_RIGHT)
        maximumBoxSizer.Add(hs, 0)
        
        hs = wx.BoxSizer(wx.HORIZONTAL)
        self.hostsNumberLabel = wx.StaticText(self, label="Number of simultaneous clients:")
        hs.Add(self.hostsNumberLabel, 0)
        self.maximumRepetitionText = wx.StaticText(self, label="")
        hs.Add(self.maximumRepetitionText, 0, wx.ALIGN_RIGHT)
        maximumBoxSizer.Add(hs, 0)
        
        self.resultsBox = wx.StaticBox(self, label="Optimization results")
        self.resultsBoxSizer = wx.StaticBoxSizer(self.resultsBox, wx.VERTICAL)
        
        processBoxSizer.Add(self.statusText, 0, wx.ALIGN_CENTER|wx.ALL, 10)
        processBoxSizer.Add(self.dotsText, 0, wx.ALIGN_CENTER|wx.ALL, 10)
        processBoxSizer.Add(self.repetitionText, 0, wx.ALIGN_CENTER|wx.ALL, 10)
        processBoxSizer.Add(maximumBoxSizer, 0, wx.EXPAND|wx.ALL, 10)
        processBoxSizer.Add(self.resultsBoxSizer, 0, wx.EXPAND|wx.ALL, 10)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(descText, 0, wx.EXPAND | wx.ALL, 10)
        sizer.Add(configurationBoxSizer, 0, wx.EXPAND|wx.ALL, 10)
        sizer.Add(processBoxSizer, 0, wx.EXPAND|wx.ALL, 10)
        
        self.SetSizer(sizer)
        self.SetScrollRate(0, 10)
        
    def _OptimizationStep(self):
        """
        Implements one step of optimization.
        """
        nextRepetition = self._GenerateNewRepetitionNumber(self.maxTime, self.previousTime, 
                                    self.currentTime, self.previousRepetition, self.currentRepetition)
        if nextRepetition is None:
            self.currentTime = None
            self.currentRepetition = None
            self._FinishSimulatorOptimization()
            return

        if nextRepetition == self.currentRepetition:
            self._FinishSimulatorOptimization()
            return

        self.previousRepetition = self.currentRepetition
        self.currentRepetition = nextRepetition
        
        simulator = self.optimizedSimulators[self.optimizedSimulatorIndex]
        newVersion = simulator.context.version.clone()
        changed = False 
        for rh in newVersion.run_hosts:
            if rh.host_name == self.hostName:
                rh.repetitions = nextRepetition
                changed = True
                break
        if not changed:
            self.currentTime = None
            self._FinishSimulatorOptimization()
            return
         
        self.repetitionText.SetLabel("Number of simultaneous hosts: %d" % nextRepetition)
        self.newSimulator = self.interpreter.builder.build_simulator(self.interpreter.store, newVersion)
        self.interpreter.install_modules(self.newSimulator)
        wx.lib.delayedresult.startWorker(self._FinishStep, 
                                         self.interpreter.run_simulation, 
                                         wargs=(self.newSimulator,))
        
    def _FinishStep(self, result):
        """
        Handles the result of one optimization step (simulation of new repetition number). 
        """
        
        try :
            result.get()
            
            self.previousTime = self.currentTime
            self.currentTime = self._GetTime(self.newSimulator, self.timeType, self.hostName)
            
            if (self.currentTime == self.previousTime) and (self.currentRepetition == self.previousRepetition):
                self._FinishSimulatorOptimization()
                return
            
            if (self.currentTime <= self.previousTime) and (self.currentRepetition >= self.previousRepetition):
                self.currentTime = None
                self.currentRepetition = None
                self._FinishSimulatorOptimization()
                return
            
            relError = abs(self.maxTime - self.currentTime)/self.maxTime
            if relError <= self.tolerance:
                self._FinishSimulatorOptimization()
                return
                
            self._OptimizationStep()
                    
        except RuntimeException, e:
            self.currentTime = None
            self._FinishSimulatorOptimization()
        except Exception, e:
            import traceback
            print traceback.format_exc()
            self.currentTime = None
            self._FinishSimulatorOptimization()
        
                
    def _FinishSimulatorOptimization(self):
        """
        Actions taken when optimization of one simulator is finished.
        """
        timeText = "Failed"
        repetitionText = ""
        
        simulator = self.optimizedSimulators[self.optimizedSimulatorIndex]
        self.optimizationResults[simulator] = (self.currentTime, self.currentRepetition)
        
        if self.currentTime is not None:
            timeText = "%.2f ms" % self.currentTime
            repetitionText = "%d" % self.currentRepetition
            
        hS = wx.BoxSizer(wx.HORIZONTAL)
        hS.Add(wx.StaticText(self, label=simulator.context.version.name, size=(200, -1)), 0)
        hS.Add(wx.StaticText(self, label=timeText, size=(200, -1)), 0)
        hS.Add(wx.StaticText(self, label=repetitionText, size=(200, -1)), 0)
        self.resultsBoxSizer.Add(hS, 0, wx.ALL | wx.EXPAND, 0)
        self.Layout()
        
        self.optimizedSimulatorIndex += 1
        self._OptimizeNextSimulator()
        
    def _OptimizeNextSimulator(self):
        """ 
        Function find the number of simultaneous clients 
        for the next simulator from class field ```optimizedSimulators```.
        The next simulator is at index ```optimizedSimulatorIndex``` field.
        When no more simulators are left the optimization process is finished.
        """
        
        if self.optimizedSimulatorIndex >= len(self.optimizedSimulators):
            self.progressTimer.Stop()
            self.startButton.Enable(True)
            
            self.statusText.SetLabel("Finished")
            self.dotsText.SetLabel("")
            self.dotsText.Hide()
            self.repetitionText.SetLabel("")
            self.repetitionText.Hide()
            
            # Create report.
            reportText = "The execution time of the scenario {0} ({1} simultaneous clients) is similar (with the {2}% tolerance): \n".format(
                                self.maxSimulator.context.version.name, self.maxRepetition, self.tolerance*100.0)
            
            simulatorsReportTexts = []
            for simulator in self.optimizedSimulators:
                results = self.optimizationResults[simulator]
                if results[0] is None:
                    continue
                    
                simulatorsReportTexts.append(" - to the scenario {0} with {1} simultaneous clients".format(
                                                simulator.context.version.name, results[1]))
                            
            reportText += " and\n".join(simulatorsReportTexts) + "."
            reportTextCtrl = wx.StaticText(self, label=reportText, style=wx.TE_MULTILINE)
            self.resultsBoxSizer.Add(reportTextCtrl, 0, wx.ALL | wx.EXPAND, 10)
            
            self.Layout()
            wx.PostEvent(self.module.get_gui(), aqopa_gui.ModuleSimulationFinishedEvent())
            return
            
        simulator = self.optimizedSimulators[self.optimizedSimulatorIndex]
        
        self.previousRepetition = 0
        self.previousTime = 0
        self.currentRepetition = self._GetHostRepetition(simulator, self.hostName)
        self.currentTime = self._GetTime(simulator, self.timeType, self.hostName)
        
        self.statusText.SetLabel("Working on %s" % simulator.context.version.name)
        self.repetitionText.SetLabel("")
        self._OptimizationStep()
        
    def _GetTime(self, simulator, timeType, hostName):
        """ 
        Returns the execution time of simulator.
        Parameter ``timeType`` selects the way of calculating time.
        It may be total execution time or average execution time of 
        host ``hostName``.
        """  
        times = self.module.current_times[simulator]
        if timeType == TIME_TYPE_TOTAL:
            return max([times[i] for i in times ])
        else: # TIME_TYPE_AVG
            nb_hosts = 0
            sum_times = 0.0
            for h in times:
                if h.original_name() == hostName:
                    nb_hosts += 1
                    sum_times += times[h]
            if nb_hosts > 0:
                return sum_times / nb_hosts
        return 0
    
    def _GetHostRepetition(self, simulator, hostName):
        """
        Returns number of repeated hosts ``hostName``.
        """
        version = simulator.context.version
        nb = 0
        for rh in version.run_hosts:
            if rh.host_name == hostName:
                nb += rh.repetitions
        return nb
    
    def _GetMaximumTimeSimulator(self, simulators, timeType, hostName):
        """ 
        Returns triple (simulator, time) with the maximum execution 
        time depending on the tipe of time selected
        """
        simulator = None
        time = 0.0
        for s in simulators:
            t = self._GetTime(s, timeType, hostName)
            if t >= time:
                simulator = s
                time = t
        return (simulator, time)
    
    def _GenerateNewRepetitionNumber(self, maxTime, previousTime, currentTime, 
                                     previousRepetition, currentRepetition):
        """
        Return the next number of simultaneous hosts closer to the final result.
        """
        if previousTime == currentTime:
            if currentRepetition == 0:
                return None
            a = currentTime / currentRepetition
            if a == 0:
                return None
            return int(maxTime / a)
        else:
            if currentRepetition == previousRepetition:
                return None
            a = (currentTime - previousTime) / (currentRepetition - previousRepetition)
            b = currentTime - currentRepetition * a
            return int((maxTime - b) / a)
        
    def RemoveAllSimulations(self):
        """ """
        self.checkBoxes = []
        self.checkBoxToSimulator = {}
        self.optimizedSimulators = []
        self.optimizationResults = {}
        self.optimizedSimulatorIndex = 0

        self.startButton.Enable(False)
        self.hostCombo.Clear()
        self.hostCombo.SetValue("")
        self.versionsSelectSizer.Clear(True)

        self.processBox.Hide()
        self.statusText.Hide()
        self.dotsText.Hide()
        self.repetitionText.Hide()

        self.maximumBox.Hide()
        self.versionsLabel.Hide()
        self.timeLabel.Hide()
        self.hostsNumberLabel.Hide()
        self.maximumRepetitionText.Hide()
        self.maximumRepetitionText.SetLabel("")
        self.maximumTimeText.Hide()
        self.maximumTimeText.SetLabel("")
        self.maximumVersionText.Hide()
        self.maximumVersionText.SetLabel("")

        self.resultsBox.Hide()
        self.resultsBoxSizer.Clear(True)

        self.Layout()

    def AddFinishedSimulation(self, simulator):
        """ """
        version = simulator.context.version
        ch = wx.CheckBox(self, label=version.name)
        self.checkBoxes.append(ch)
        self.checkBoxToSimulator[ch] = simulator
        self.versionsSelectSizer.Add(ch)
        self.Layout()
    
    def OnAllSimulationsFinished(self, simulators):
        """ """
        items = []
        for s in simulators:
            version = s.context.version
            for rh in version.run_hosts:
                if rh.host_name not in items:
                    items.append(rh.host_name)

        self.hostCombo.Clear()
        self.hostCombo.AppendItems(items)
        self.hostCombo.Select(0)

        self.startButton.Enable(True)
        self.Layout()
    
    def OnStartClick(self, event=None):
        """ """
        self.processBox.Show()
        self.statusText.Show()
        self.dotsText.Show()
        self.repetitionText.Show()

        self.maximumBox.Show()
        self.versionsLabel.Show()
        self.timeLabel.Show()
        self.hostsNumberLabel.Show()
        self.maximumRepetitionText.Show()
        self.maximumTimeText.Show()
        self.maximumVersionText.Show()

        self.resultsBox.Show()

        simulators = []
        for ch in self.checkBoxes:
            if ch.IsChecked():
                simulators.append(self.checkBoxToSimulator[ch])
                
        if len(simulators) < 2:
            wx.MessageBox('Please select at least 2 versions.', 'Error', 
                          wx.OK | wx.ICON_ERROR)
            return
        
        try:
            self.tolerance = float(self.toleranceTextCtrl.GetValue())/100.0 
        except ValueError:
            wx.MessageBox('Tolerance must be a float number.', 'Error', 
                          wx.OK | wx.ICON_ERROR)
            return
        
        if self.tolerance >= 1:
            wx.MessageBox('Tolerance must be smaller than 100%.', 'Error', 
                          wx.OK | wx.ICON_ERROR)
            return
        
        if self.tolerance <= 0.01:
            wx.MessageBox('Tolerance must be greater than 1%.', 'Error', 
                          wx.OK | wx.ICON_ERROR)
            return
        
        self.timeType = TIME_TYPE_TOTAL
        if self.avgRadioBtn.GetValue():
            self.timeType = TIME_TYPE_AVG
        self.hostName = self.hostCombo.GetValue() 
        self.maxSimulator, self.maxTime = self._GetMaximumTimeSimulator(simulators, self.timeType, self.hostName)
        self.maxRepetition = self._GetHostRepetition(self.maxSimulator, self.hostName)
        
        self.maximumVersionText.SetLabel("%s" % self.maxSimulator.context.version.name)
        self.maximumTimeText.SetLabel("%.5f ms" % self.maxTime)
        self.maximumRepetitionText.SetLabel("%d" % self.maxRepetition)
        
        if self.maxTime == 0:
            wx.MessageBox('Maximum time is equal to 0. Cannot optimize.', 'Error', 
                          wx.OK | wx.ICON_ERROR)
            return
        
        self.startButton.Enable(False)
        self.statusText.SetLabel("Waiting for simulator")
        
        self.resultsBoxSizer.Clear(True)
        hS = wx.BoxSizer(wx.HORIZONTAL)
        hS.Add(wx.StaticText(self, label="Version", size=(200, -1)), 0)
        hS.Add(wx.StaticText(self, label="Time", size=(200, -1)), 0)
        hS.Add(wx.StaticText(self, label="Number of simulatenous hosts", size=(200, -1)), 0)
        self.resultsBoxSizer.Add(hS, 0, wx.ALL | wx.EXPAND, 0)
        
        self.Layout()
        
        simulators.remove(self.maxSimulator)
        self.progressTimer.Start(500)
        self.optimizedSimulators = simulators
        self.optimizedSimulatorIndex = 0
        
        wx.PostEvent(self.module.get_gui(), aqopa_gui.ModuleSimulationRequestEvent(module=self.module))
          
    def OnSimulationAllowed(self, event):
        """ """
        self.interpreter = event.interpreter
        self._OptimizeNextSimulator()
            
    ################
    # PROGRESS BAR 
    ################
        
    def OnProgressTimerTick(self, event):
        """ """
        self.dots  = (self.dots + 1) % 10
        self.dotsText.SetLabel("." * self.dots)
        self.Layout()
        
class MainResultsNotebook(wx.Notebook):
    """ """
    def __init__(self, module, *args, **kwargs):
        wx.Notebook.__init__(self, *args, **kwargs)
        
        self.module = module
        
        self.oneVersionTab = SingleVersionPanel(self.module, self)
        self.AddPage(self.oneVersionTab, "Single Version")
        self.oneVersionTab.Layout()
        
#        self.compareTab = VersionsChartsPanel(self.module, self)
#        self.AddPage(self.compareTab, "Versions' Charts")
#        self.compareTab.Layout()
        
        self.distributedOptimizationTab = DistributedSystemOptimizationPanel(self.module, self)
        self.AddPage(self.distributedOptimizationTab, "Distributed System Optimization")
        self.distributedOptimizationTab.Layout()
        
    def OnParsedModel(self):
        """ """
        self.oneVersionTab.RemoveAllSimulations()
#        self.compareTab.RemoveAllSimulations()
        self.distributedOptimizationTab.RemoveAllSimulations()
        
    def OnSimulationFinished(self, simulator):
        """ """
        self.oneVersionTab.AddFinishedSimulation(simulator)
#        self.compareTab.AddFinishedSimulation(simulator)
        self.distributedOptimizationTab.AddFinishedSimulation(simulator)
        
    def OnAllSimulationsFinished(self, simulators):
        """ """
        self.distributedOptimizationTab.OnAllSimulationsFinished(simulators)
        
class ModuleGui(wx.EvtHandler):
    """
    Class used by GUI version of AQoPA.
    """
    
    def __init__(self, module):
        """ """
        wx.EvtHandler.__init__(self)
        
        self.module = module
        self.mainResultNotebook = None
        
    def get_name(self):
        return "Time Analysis"
    
    def get_configuration_panel(self, parent):
        """ Returns WX panel with configuration controls. """
        
        panel = wx.Panel(parent)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        text = wx.StaticText(panel, label="Module does not need to be configured.") 
        sizer.Add(text, 0, wx.ALL | wx.EXPAND, 5)
        text = wx.StaticText(panel, label="All result options will be available after results are calculated.") 
        sizer.Add(text, 0, wx.ALL | wx.EXPAND, 5)
        
        panel.SetSizer(sizer)
        return panel
    
    def get_results_panel(self, parent):
        """
        Create main result panel existing from the beginning 
        which will be extended when versions' simulations are finished.
        """
        self.mainResultNotebook = MainResultsNotebook(self.module, parent)
        return self.mainResultNotebook
    
    def on_finished_simulation(self, simulator):
        """ """
        self.mainResultNotebook.OnSimulationFinished(simulator)
    
    def on_finished_all_simulations(self, simulators):
        """ 
        Called once for all simulations after all of them are finished.
        """
        self.mainResultNotebook.OnAllSimulationsFinished(simulators)
        
    def on_parsed_model(self):
        """ """
        self.mainResultNotebook.OnParsedModel()
        
        

        
        
        
        



'''
Created on 07-05-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''
import copy
from aqopa.model import IdentifierExpression, CallFunctionExpression, TupleExpression,\
    BooleanExpression, ComparisonExpression, TupleElementExpression
from aqopa.simulator.error import RuntimeException


class Populator():
    
    def populate(self, expression, variables, reducer):
        """
        Returns new expression with replaced variables names 
        with copies of values of variables from variables list.
        Reducer is used for special types of variables, 
        eg. tuple element expressions.
        """
        if isinstance(expression, IdentifierExpression):
            if expression.identifier not in variables:
                raise RuntimeException("Variable '%s' does not exist." % expression.identifier)
            return variables[expression.identifier].clone()
            
        if isinstance(expression, CallFunctionExpression):
            arguments = []
            for arg in expression.arguments:
                arguments.append(self.populate(arg, variables, reducer))
            qop_arguments = []
            for qop_arg in expression.qop_arguments:
                qop_arguments.append(qop_arg)
            return CallFunctionExpression(expression.function_name, arguments, qop_arguments)
            
        if isinstance(expression, TupleExpression):
            elements = []
            for e in expression.elements:
                elements.append(self.populate(e, variables, reducer))
            return TupleExpression(elements)
        
        if isinstance(expression, TupleElementExpression):
            if expression.variable_name not in variables:
                raise RuntimeException("Variable '%s' does not exist." % expression.variable_name)
            expr = variables[expression.variable_name]
            if not isinstance(expr, TupleExpression):
                expr = reducer.reduce(expr)
            if not isinstance(expr, TupleExpression):
                raise RuntimeException("Cannot compute expression %s. Variable '%s' is not a tuple. It is: %s."
                                       % (unicode(expression), expression.variable_name, unicode(expr)))
            if len(expr.elements) <= expression.index:
                raise RuntimeException(
                    "Cannot compute expression %s. Variable '%s' does not have index %s. It has %d elements."
                    % (unicode(expression), expression.variable_name, expression.index, len(expr.elements)))
            return self.populate(expr.elements[expression.index], variables, reducer)
        
        return expression.clone()


class Checker():
    """
    Expression checker.
    Class used to check the result of expressions.
    """
    
    def _are_equal(self, left, right):
        """
        Method checks if both expressions are the same.
        
        Expressions cannot contains identifiers. All identifiers should be replaced with its values.
        Checks whether all function calls and boolean expressions have the same values,
        function names and parameters' values.
        """
        if left.__class__ != right.__class__:
            return False
        
        if isinstance(left, BooleanExpression):
            return left.val == right.val

        if isinstance(left, CallFunctionExpression):
            if left.function_name != right.function_name:
                return False
            if len(left.arguments) != len(right.arguments):
                return False
            for i in range(0, len(left.arguments)):
                if not self._are_equal(left.arguments[i], right.arguments[i]):
                    return False
        return True
    
    def result(self, condition, variables, functions, populator, reducer):
        """
        Method checks the result of condition.
        Returns True if condition is true or can be reduced to true condition.
        
        At the moment only comparision condition is available.  
        """
        
        if isinstance(condition, BooleanExpression):
            return condition.is_true()
        
        if isinstance(condition, ComparisonExpression):
            left = condition.left
            right = condition.right
            
            left = populator.populate(left, variables, reducer)
            right = populator.populate(right, variables, reducer)
            
            left = reducer.reduce(left)
            right = reducer.reduce(right)

            result = self._are_equal(left, right)
            del left
            del right
            return result
        
        return False

class ReductionPoint():
    """
    Class representing point where expression can be reduced.
    """
    
    def __init__(self, equation, expression, replacement, modified_part=None, modified_element_info=None):
        
        self.equation       = equation      # Equation that is used to reduce
        self.expression     = expression    # Expression that will be reduced
                                            # Reduction can replace whole expression 
                                            # or the part of expression
        self.replacement    = replacement   # Expression that will replace reduced part
        self.replaced = None                # Part of expression that was replaced with replacement
                                
        self.modified_part          = modified_part         # Part of expression that will be modified
                                                            # If None, whole expression should be replaced
        self.modified_element_info  = modified_element_info # Information which element 
                                                            # of modified part should be replaced
    
    def equals_to(self, reduction_point):
        """
        Returns True if self and reduction_points try to reduct the expression in the same place.
        """
        return self.expression == reduction_point.expression \
            and self.modified_part == reduction_point.modified_part \
            and self.modified_element_info == reduction_point.modified_element_info
        
    def _get_replaced_part(self):
        """
        Returns the part of expression that will be replaced.
        """
        if isinstance(self.modified_part, CallFunctionExpression):
            return self.modified_part.arguments[self.modified_element_info]
        
    def _replace_part(self, replacement):
        """
        Replace the part of modified_part according to modified_element_info.
        """
        if isinstance(self.modified_part, CallFunctionExpression):
            self.modified_part.arguments[self.modified_element_info] = replacement
        
    def reduce(self):
        """
        Returns reduced expression. 
        Method saves informaction for rolling back the reduction.
        """ 
        if self.modified_part is None:
            return self.replacement 
        
        self.replaced = self._get_replaced_part()
        self._replace_part(self.replacement)
        return self.expression
        
    def rollback(self):
        """
        Rolls back the reduction. 
        Returns expression to the form before reduction. 
        """
        if self.modified_part is None:
            return self.expression
        
        self._replace_part(self.replaced)
        self.replaced = None
        return self.expression

class Reducer():
    """
    Expression reducer.
    Class used to recude complex expressions with usage of equations.
    """
    
    def __init__(self, equations):
        self.equations = equations
        
        
    def _get_reduction_points_for_equation(self, equation, whole_expression, current_expression, 
                                           parent_expression=None, modified_element_info=None):
        """
        Recursive strategy of finding points.
        """
        points = []
        variables = {}
        
        if equation._can_reduce(current_expression, equation.composite, variables):
            
            if isinstance(equation.simple, IdentifierExpression):
                simpler_value = variables[equation.simple.identifier]
            else:
                simpler_value = equation.simple
                
            points.append(ReductionPoint(equation, 
                                         expression=whole_expression, 
                                         replacement=simpler_value, 
                                         modified_part=parent_expression, 
                                         modified_element_info=modified_element_info))
        
        if isinstance(current_expression, CallFunctionExpression):
            
            for i in range(0, len(current_expression.arguments)):
                e = current_expression.arguments[i]
                for p in self._get_reduction_points_for_equation(equation, 
                                                                 whole_expression, 
                                                                 current_expression=e,
                                                                 parent_expression=current_expression,
                                                                 modified_element_info=i):
                    points.append(p)
        
        return points
        
    def get_reduction_points_for_equation(self, equation, expression):
        """
        Method returns list of points where equation can reduce expression.
        
        Example:
        equation:   f(x) = x
        expression: f(a(f(b())))
                    ^   ^
        Two reduction points selected above,
        """
        return self._get_reduction_points_for_equation(equation=equation, 
                                          whole_expression=expression, 
                                          current_expression=expression,
                                          parent_expression=None,
                                          modified_element_info=None)
        
    def _get_reduction_points(self, expression):
        """
        Method finds points in expression where it can be reduced.
        """
        points = []
        for eq in self.equations:
            for p in self.get_reduction_points_for_equation(eq, expression):
                points.append(p)
        return points
        
    def reduce(self, expression):
        """
        Reduces expression with usage of equations.
        Returns reduced expression.
        Raises exception if ambiguity is found.
        """
        continue_reducing = True
   
        """     
        # Wrap expression and user wrapper variable
        # Used to simulate real pass-by-reference
        # If expression was passed without wrapped variable and equation wanted to
        # replace whole exception, it would not work, because whole variable cannot be changed
        # For example:
        # eq enc(K, dec(K, X)) = X
        # expression: enc(k(), dec(k(), b()) should be reduced to b()
        # If we pass whole expression variable to reduce we cannot change it whole
        # def f(x,v) -> x = v
        # e = A(a=1)
        # f(e, A(a=2)) - will not work, because e is reference to instance, but passed by value
        # When we pass wrapper, we can change its element (ie. expression)
        """
        
        # Reduce until no reduction can be performed.
        # One reduction can give way for another reduction.
        while continue_reducing:
            continue_reducing = False
            
            # For each equation we find points where expression can be reduced
            reduction_points = self._get_reduction_points(expression)
        
            if len(reduction_points) > 0:
                
                # For each point:
                #  - temporary reduce at this point
                #  - remove used point from reduction points list
                #  - generate new reduction points list for reduced expression
                #  - if any of points from old list is not present in new list raise ambiguity exception
                #  ! New reduction points may come
                
                for reduction_point in reduction_points:
                    
                    # Generate list with reduction points before reduction
                    old_reduction_points = copy.copy(reduction_points)
                    old_reduction_points.remove(reduction_point)
                    
                    # Reduce temporary
                    expression = reduction_point.reduce()
                    
                    # Generate new reduction points
                    new_reduction_points = self._get_reduction_points(expression)
                    
                    for old_reduction_point in old_reduction_points:
                        found = False
                        for new_reduction_point in new_reduction_points:
                            if old_reduction_point.equals_to(new_reduction_point):
                                found = True
                                break
                        if not found:
                            raise RuntimeException("Equations '%s' and '%s are ambiguous for expression: %s." % 
                                                   (unicode(old_reduction_point.equation),
                                                    unicode(reduction_point.equation),
                                                    unicode(expression)))
                            
                    # Cancel reduction
                    expression = reduction_point.rollback()
                    
                # Ambiguity error checked
                    
                # Make factual reduction
                reduction_point = reduction_points[0]
                
                # Reduce and commit reduction
                expression = reduction_point.reduce()
                
                continue_reducing = True
             
            return expression
                
                
                
                
                
                


'''
Created on 14-05-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''
from aqopa.simulator.error import RuntimeException
from aqopa.model import TupleExpression, CallFunctionExpression, CallFunctionInstruction, IdentifierExpression, TupleElementExpression


class HostMetrics():
    """
    Whole metrics configuration for particular hosts.
    Configuration includes the name, the device configuration params,
    three types of metrics: normal, plus and star.
    """
    def __init__(self, name):
        self.name = name
        self.configurations = []
        
        self.normal_blocks = []
        self.plus_blocks = []
        self.star_blocks = []
        
        self.connected_hosts = []
        
    def is_connected_with_host(self, host):
        """ """
        return host in self.connected_hosts

class Block():
    """
    Block of metrics containing all metrics with the same params 
    and service params definition.
    """
    def __init__(self, params, service_params):
        self.params = params
        self.service_params = service_params
        self.metrics = []

    def add_metric(self, metric):
        """ """
        self.metrics.append(metric)
        metric.block = self
        
    def find_primitives(self, function_name, arguments):
        """ """
        if len(arguments) != len(self.params):
            return []
        
        found_metrics = []
        for m in self.metrics:
            if m.function_name != function_name:
                continue
            
            params_ok = True
            for i in range(0, len(arguments)):
                if arguments[i] != m.arguments[i]:
                    params_ok = False
                    
            if not params_ok:
                continue
            
            found_metrics.append(m)
            
        return found_metrics


class Metric():
    """
    One metrics row with defined: call arguments and service arguments.
    Call arguments are used for metric lookup and service arguments are used
    to define new state or by modules to calculate some informations.
    """
    def __init__(self, function_name, arguments, service_arguments):
        self.function_name = function_name
        self.arguments = arguments
        self.service_arguments = service_arguments   
        self.block = None 

class Manager():
    """
    Metrics manager. 
    Class used for operations on metrics: searching, etc.
    """
    def __init__(self, host_metrics):
        self.host_metrics = host_metrics
        
    def find_primitive(self, host, call_function_expression):
        """
        Method finds metric for function call in host.
        Method searches the metric in three kinds: normal, plus and star.
        If more than one metrics is found, runtime exception is raised. 
        """
        found_host_metric = None
        for host_metric in self.host_metrics:
            if host_metric.is_connected_with_host(host):
                found_host_metric = host_metric
                break
            
        if not found_host_metric:
            return None
        
        normal_metrics = []
        plus_metrics = []
        star_metrics = []

        if len(found_host_metric.normal_blocks) > 0:
            for mb in found_host_metric.normal_blocks:
                if len(mb.params) == len(call_function_expression.qop_arguments):
                    normal_metrics.extend(mb.find_primitives(
                                        call_function_expression.function_name,
                                        call_function_expression.qop_arguments))
        
        if len(found_host_metric.plus_blocks) > 0:
            for mb in found_host_metric.plus_blocks:
                if len(mb.params) == len(call_function_expression.qop_arguments):
                    plus_metrics.extend(mb.find_primitives(
                                        call_function_expression.function_name,
                                        call_function_expression.qop_arguments))
        
        if len(found_host_metric.star_blocks) > 0:
            for mb in found_host_metric.star_blocks:
                if len(mb.params) == len(call_function_expression.qop_arguments):
                    star_metrics.extend(mb.find_primitives(
                                        call_function_expression.function_name,
                                        call_function_expression.qop_arguments))
                
        if len(normal_metrics) + len(plus_metrics) + len(star_metrics) > 1:
            raise RuntimeException("Found many metrics for function '%s' with qop arguments: %s." \
                                   % (call_function_expression.function_name,
                                      ', '.join(call_function_expression.qop_arguments)))
        
        if len(normal_metrics) > 0:
            return normal_metrics[0]
        if len(plus_metrics) > 0:
            return plus_metrics[0]
        if len(star_metrics) > 0:
            return star_metrics[0]
        return None
        
        
    def get_expression_size(self, expression, context, host):
        """
        Returns the size in bytes of expression according to metrics.
        Metrics van specify exact size (ie. in bytes, bits, kilobytes, kilobits, megabytes, megabits)
        or ratio size (ie. 0.5 equals 50%) used ie. in compression.
        Expression cannot have variables - it must be filled with variables' values.
        """
        size = 0

        if isinstance(expression, IdentifierExpression):
            # If expression is an variable, get its value size
            return self.get_expression_size(host.get_variable(expression.identifier), context, host)

        if isinstance(expression, TupleElementExpression):
            variable = host.get_variable(expression.variable_name)
            variable = context.expression_reducer.reduce(variable)
            if not isinstance(variable, TupleExpression):
                raise RuntimeException('Cannot get tuple element on expression: %s.' % unicode(variable))
            return self.get_expression_size(variable.elements[expression.index], context, host)

        elif isinstance(expression, TupleExpression):
            # If expression is tuple, just sum its elements' sizes
            for expr in expression.elements:
                size += self.get_expression_size(expr, context, host)
            return size    
            
        if isinstance(expression, CallFunctionExpression) or isinstance(expression, CallFunctionInstruction):
            metric = self.find_primitive(host, expression)
            
            if not metric:
                raise RuntimeException("Cannot get expression size: No metric found for expression '%s'." 
                                       % unicode(expression))

            block = metric.block
            for i in range(0, len(block.service_params)):
                sparam = block.service_params[i]

                if sparam.service_name.lower() != "size":
                    continue
                
                metric_type = sparam.param_name.lower()
                metric_unit = sparam.unit
                metric_value = metric.service_arguments[i]
            
                if metric_type == "ratio":
                    mparts = metric_value.split(':')
                    element_index = int(mparts[0])-1
                    percent = float(mparts[1])
                    
                    size = self.get_expression_size(expression.arguments[element_index], context, host) \
                            * percent
                            
                elif metric_type == "exact":
                    if metric_unit == 'B':
                        size = float(metric_value)
                    elif metric_unit == 'b':
                        size = float(metric_value)/8.0
                    else:
                        raise RuntimeException('Cannot get expression size: Unsupported size value for exact type.')

                elif metric_type == "block":
                    mparts = metric_value.split(':')
                    element_index = int(mparts[0])-1
                    unit_value = int(mparts[1])

                    factor = 1.0
                    if metric_unit == 'b':
                        factor = 1.0 / 8.0

                    argument_size = self.get_expression_size(expression.arguments[element_index], context, host)
                    argument_size_excess = argument_size % unit_value

                    size = argument_size
                    if argument_size_excess > 0:
                        size += unit_value - argument_size_excess
                    size *= factor

                elif metric_type == "nested":
                    mparts = metric_value.split(':')
                    element_index = int(mparts[0])-1
                    nested_element_index = int(mparts[1])-1

                    nested_expression = expression.arguments[element_index]
                    if not isinstance(expression, CallFunctionExpression) and \
                        not isinstance(expression, CallFunctionInstruction):
                        raise RuntimeException('Cannot get nested expression size: Not a function call.')

                    size = self.get_expression_size(nested_expression.arguments[nested_element_index], context, host)

                else:
                    raise RuntimeException('Cannot get expression size: Unsupported size type.')

                return size
            
        raise RuntimeException('Cannot get expression size: Unsupported expression type.')


from __future__ import division, absolute_import, print_function

import re
import os
import sys
import warnings
import platform
import tempfile
from subprocess import Popen, PIPE, STDOUT

from numpy.distutils.cpuinfo import cpu
from numpy.distutils.fcompiler import FCompiler
from numpy.distutils.exec_command import exec_command
from numpy.distutils.misc_util import msvc_runtime_library
from numpy.distutils.compat import get_exception

compilers = ['GnuFCompiler', 'Gnu95FCompiler']

TARGET_R = re.compile("Target: ([a-zA-Z0-9_\-]*)")

# XXX: handle cross compilation
def is_win64():
    return sys.platform == "win32" and platform.architecture()[0] == "64bit"

if is_win64():
    #_EXTRAFLAGS = ["-fno-leading-underscore"]
    _EXTRAFLAGS = []
else:
    _EXTRAFLAGS = []

class GnuFCompiler(FCompiler):
    compiler_type = 'gnu'
    compiler_aliases = ('g77',)
    description = 'GNU Fortran 77 compiler'

    def gnu_version_match(self, version_string):
        """Handle the different versions of GNU fortran compilers"""
        m = re.match(r'GNU Fortran', version_string)
        if not m:
            return None
        m = re.match(r'GNU Fortran\s+95.*?([0-9-.]+)', version_string)
        if m:
            return ('gfortran', m.group(1))
        m = re.match(r'GNU Fortran.*?\-?([0-9-.]+)', version_string)
        if m:
            v = m.group(1)
            if v.startswith('0') or v.startswith('2') or v.startswith('3'):
                # the '0' is for early g77's
                return ('g77', v)
            else:
                # at some point in the 4.x series, the ' 95' was dropped
                # from the version string
                return ('gfortran', v)

    def version_match(self, version_string):
        v = self.gnu_version_match(version_string)
        if not v or v[0] != 'g77':
            return None
        return v[1]

    # 'g77 --version' results
    # SunOS: GNU Fortran (GCC 3.2) 3.2 20020814 (release)
    # Debian: GNU Fortran (GCC) 3.3.3 20040110 (prerelease) (Debian)
    #         GNU Fortran (GCC) 3.3.3 (Debian 20040401)
    #         GNU Fortran 0.5.25 20010319 (prerelease)
    # Redhat: GNU Fortran (GCC 3.2.2 20030222 (Red Hat Linux 3.2.2-5)) 3.2.2 20030222 (Red Hat Linux 3.2.2-5)
    # GNU Fortran (GCC) 3.4.2 (mingw-special)

    possible_executables = ['g77', 'f77']
    executables = {
        'version_cmd'  : [None, "--version"],
        'compiler_f77' : [None, "-g", "-Wall", "-fno-second-underscore"],
        'compiler_f90' : None, # Use --fcompiler=gnu95 for f90 codes
        'compiler_fix' : None,
        'linker_so'    : [None, "-g", "-Wall"],
        'archiver'     : ["ar", "-cr"],
        'ranlib'       : ["ranlib"],
        'linker_exe'   : [None, "-g", "-Wall"]
        }
    module_dir_switch = None
    module_include_switch = None

    # Cygwin: f771: warning: -fPIC ignored for target (all code is
    # position independent)
    if os.name != 'nt' and sys.platform != 'cygwin':
        pic_flags = ['-fPIC']

    # use -mno-cygwin for g77 when Python is not Cygwin-Python
    if sys.platform == 'win32':
        for key in ['version_cmd', 'compiler_f77', 'linker_so', 'linker_exe']:
            executables[key].append('-mno-cygwin')

    g2c = 'g2c'

    suggested_f90_compiler = 'gnu95'

    #def get_linker_so(self):
    #    # win32 linking should be handled by standard linker
    #    # Darwin g77 cannot be used as a linker.
    #    #if re.match(r'(darwin)', sys.platform):
    #    #    return
    #    return FCompiler.get_linker_so(self)

    def get_flags_linker_so(self):
        opt = self.linker_so[1:]
        if sys.platform=='darwin':
            target = os.environ.get('MACOSX_DEPLOYMENT_TARGET', None)
            # If MACOSX_DEPLOYMENT_TARGET is set, we simply trust the value
            # and leave it alone.  But, distutils will complain if the
            # environment's value is different from the one in the Python
            # Makefile used to build Python.  We let disutils handle this
            # error checking.
            if not target:
                # If MACOSX_DEPLOYMENT_TARGET is not set in the environment,
                # we try to get it first from the Python Makefile and then we
                # fall back to setting it to 10.3 to maximize the set of
                # versions we can work with.  This is a reasonable default
                # even when using the official Python dist and those derived
                # from it.
                import distutils.sysconfig as sc
                g = {}
                filename = sc.get_makefile_filename()
                sc.parse_makefile(filename, g)
                target = g.get('MACOSX_DEPLOYMENT_TARGET', '10.3')
                os.environ['MACOSX_DEPLOYMENT_TARGET'] = target
                if target == '10.3':
                    s = 'Env. variable MACOSX_DEPLOYMENT_TARGET set to 10.3'
                    warnings.warn(s)

            opt.extend(['-undefined', 'dynamic_lookup', '-bundle'])
        else:
            opt.append("-shared")
        if sys.platform.startswith('sunos'):
            # SunOS often has dynamically loaded symbols defined in the
            # static library libg2c.a  The linker doesn't like this.  To
            # ignore the problem, use the -mimpure-text flag.  It isn't
            # the safest thing, but seems to work. 'man gcc' says:
            # ".. Instead of using -mimpure-text, you should compile all
            #  source code with -fpic or -fPIC."
            opt.append('-mimpure-text')
        return opt

    def get_libgcc_dir(self):
        status, output = exec_command(self.compiler_f77 +
                                      ['-print-libgcc-file-name'],
                                      use_tee=0)
        if not status:
            return os.path.dirname(output)
        return None

    def get_library_dirs(self):
        opt = []
        if sys.platform[:5] != 'linux':
            d = self.get_libgcc_dir()
            if d:
                # if windows and not cygwin, libg2c lies in a different folder
                if sys.platform == 'win32' and not d.startswith('/usr/lib'):
                    d = os.path.normpath(d)
                    if not os.path.exists(os.path.join(d, "lib%s.a" % self.g2c)):
                        d2 = os.path.abspath(os.path.join(d,
                                                          '../../../../lib'))
                        if os.path.exists(os.path.join(d2, "lib%s.a" % self.g2c)):
                            opt.append(d2)
                opt.append(d)
        return opt

    def get_libraries(self):
        opt = []
        d = self.get_libgcc_dir()
        if d is not None:
            g2c = self.g2c + '-pic'
            f = self.static_lib_format % (g2c, self.static_lib_extension)
            if not os.path.isfile(os.path.join(d, f)):
                g2c = self.g2c
        else:
            g2c = self.g2c

        if g2c is not None:
            opt.append(g2c)
        c_compiler = self.c_compiler
        if sys.platform == 'win32' and c_compiler and \
               c_compiler.compiler_type=='msvc':
            # the following code is not needed (read: breaks) when using MinGW
            # in case want to link F77 compiled code with MSVC
            opt.append('gcc')
            runtime_lib = msvc_runtime_library()
            if runtime_lib:
                opt.append(runtime_lib)
        if sys.platform == 'darwin':
            opt.append('cc_dynamic')
        return opt

    def get_flags_debug(self):
        return ['-g']

    def get_flags_opt(self):
        v = self.get_version()
        if v and v<='3.3.3':
            # With this compiler version building Fortran BLAS/LAPACK
            # with -O3 caused failures in lib.lapack heevr,syevr tests.
            opt = ['-O2']
        else:
            opt = ['-O3']
        opt.append('-funroll-loops')
        return opt

    def _c_arch_flags(self):
        """ Return detected arch flags from CFLAGS """
        from distutils import sysconfig
        try:
            cflags = sysconfig.get_config_vars()['CFLAGS']
        except KeyError:
            return []
        arch_re = re.compile(r"-arch\s+(\w+)")
        arch_flags = []
        for arch in arch_re.findall(cflags):
            arch_flags += ['-arch', arch]
        return arch_flags

    def get_flags_arch(self):
        return []

class Gnu95FCompiler(GnuFCompiler):
    compiler_type = 'gnu95'
    compiler_aliases = ('gfortran',)
    description = 'GNU Fortran 95 compiler'

    def version_match(self, version_string):
        v = self.gnu_version_match(version_string)
        if not v or v[0] != 'gfortran':
            return None
        v = v[1]
        if v>='4.':
            # gcc-4 series releases do not support -mno-cygwin option
            pass
        else:
            # use -mno-cygwin flag for gfortran when Python is not Cygwin-Python
            if sys.platform == 'win32':
                for key in ['version_cmd', 'compiler_f77', 'compiler_f90',
                            'compiler_fix', 'linker_so', 'linker_exe']:
                    self.executables[key].append('-mno-cygwin')
        return v

    # 'gfortran --version' results:
    # XXX is the below right?
    # Debian: GNU Fortran 95 (GCC 4.0.3 20051023 (prerelease) (Debian 4.0.2-3))
    #         GNU Fortran 95 (GCC) 4.1.2 20061115 (prerelease) (Debian 4.1.1-21)
    # OS X: GNU Fortran 95 (GCC) 4.1.0
    #       GNU Fortran 95 (GCC) 4.2.0 20060218 (experimental)
    #       GNU Fortran (GCC) 4.3.0 20070316 (experimental)

    possible_executables = ['gfortran', 'f95']
    executables = {
        'version_cmd'  : ["<F90>", "--version"],
        'compiler_f77' : [None, "-Wall", "-ffixed-form",
                          "-fno-second-underscore"] + _EXTRAFLAGS,
        'compiler_f90' : [None, "-Wall", "-fno-second-underscore"] + _EXTRAFLAGS,
        'compiler_fix' : [None, "-Wall", "-ffixed-form",
                          "-fno-second-underscore"] + _EXTRAFLAGS,
        'linker_so'    : ["<F90>", "-Wall"],
        'archiver'     : ["ar", "-cr"],
        'ranlib'       : ["ranlib"],
        'linker_exe'   : [None, "-Wall"]
        }

    module_dir_switch = '-J'
    module_include_switch = '-I'

    g2c = 'gfortran'

    def _universal_flags(self, cmd):
        """Return a list of -arch flags for every supported architecture."""
        if not sys.platform == 'darwin':
            return []
        arch_flags = []
        # get arches the C compiler gets.
        c_archs = self._c_arch_flags()
        if "i386" in c_archs:
            c_archs[c_archs.index("i386")] = "i686"
        # check the arches the Fortran compiler supports, and compare with
        # arch flags from C compiler
        for arch in ["ppc", "i686", "x86_64", "ppc64"]:
            if _can_target(cmd, arch) and arch in c_archs:
                arch_flags.extend(["-arch", arch])
        return arch_flags

    def get_flags(self):
        flags = GnuFCompiler.get_flags(self)
        arch_flags = self._universal_flags(self.compiler_f90)
        if arch_flags:
            flags[:0] = arch_flags
        return flags

    def get_flags_linker_so(self):
        flags = GnuFCompiler.get_flags_linker_so(self)
        arch_flags = self._universal_flags(self.linker_so)
        if arch_flags:
            flags[:0] = arch_flags
        return flags

    def get_library_dirs(self):
        opt = GnuFCompiler.get_library_dirs(self)
        if sys.platform == 'win32':
            c_compiler = self.c_compiler
            if c_compiler and c_compiler.compiler_type == "msvc":
                target = self.get_target()
                if target:
                    d = os.path.normpath(self.get_libgcc_dir())
                    root = os.path.join(d, os.pardir, os.pardir, os.pardir, os.pardir)
                    mingwdir = os.path.normpath(os.path.join(root, target, "lib"))
                    full = os.path.join(mingwdir, "libmingwex.a")
                    if os.path.exists(full):
                        opt.append(mingwdir)
        return opt

    def get_libraries(self):
        opt = GnuFCompiler.get_libraries(self)
        if sys.platform == 'darwin':
            opt.remove('cc_dynamic')
        if sys.platform == 'win32':
            c_compiler = self.c_compiler
            if c_compiler and c_compiler.compiler_type == "msvc":
                if "gcc" in opt:
                    i = opt.index("gcc")
                    opt.insert(i+1, "mingwex")
                    opt.insert(i+1, "mingw32")
            # XXX: fix this mess, does not work for mingw
            if is_win64():
                c_compiler = self.c_compiler
                if c_compiler and c_compiler.compiler_type == "msvc":
                    return []
                else:
                    raise NotImplementedError("Only MS compiler supported with gfortran on win64")
        return opt

    def get_target(self):
        status, output = exec_command(self.compiler_f77 +
                                      ['-v'],
                                      use_tee=0)
        if not status:
            m = TARGET_R.search(output)
            if m:
                return m.group(1)
        return ""

    def get_flags_opt(self):
        if is_win64():
            return ['-O0']
        else:
            return GnuFCompiler.get_flags_opt(self)

def _can_target(cmd, arch):
    """Return true is the command supports the -arch flag for the given
    architecture."""
    newcmd = cmd[:]
    fid, filename = tempfile.mkstemp(suffix=".f")
    try:
        d = os.path.dirname(filename)
        output = os.path.splitext(filename)[0] + ".o"
        try:
            newcmd.extend(["-arch", arch, "-c", filename])
            p = Popen(newcmd, stderr=STDOUT, stdout=PIPE, cwd=d)
            p.communicate()
            return p.returncode == 0
        finally:
            if os.path.exists(output):
                os.remove(output)
    finally:
        os.remove(filename)
    return False

if __name__ == '__main__':
    from distutils import log
    log.set_verbosity(2)

    compiler = GnuFCompiler()
    compiler.customize()
    print(compiler.get_version())

    try:
        compiler = Gnu95FCompiler()
        compiler.customize()
        print(compiler.get_version())
    except Exception:
        msg = get_exception()
        print(msg)



from __future__ import division, absolute_import, print_function

from numpy.testing import *
from numpy import logspace, linspace

class TestLogspace(TestCase):
    def test_basic(self):
        y = logspace(0, 6)
        assert_(len(y)==50)
        y = logspace(0, 6, num=100)
        assert_(y[-1] == 10**6)
        y = logspace(0, 6, endpoint=0)
        assert_(y[-1] < 10**6)
        y = logspace(0, 6, num=7)
        assert_array_equal(y, [1, 10, 100, 1e3, 1e4, 1e5, 1e6])

class TestLinspace(TestCase):
    def test_basic(self):
        y = linspace(0, 10)
        assert_(len(y)==50)
        y = linspace(2, 10, num=100)
        assert_(y[-1] == 10)
        y = linspace(2, 10, endpoint=0)
        assert_(y[-1] < 10)

    def test_corner(self):
        y = list(linspace(0, 1, 1))
        assert_(y == [0.0], y)
        y = list(linspace(0, 1, 2.5))
        assert_(y == [0.0, 1.0])

    def test_type(self):
        t1 = linspace(0, 1, 0).dtype
        t2 = linspace(0, 1, 1).dtype
        t3 = linspace(0, 1, 2).dtype
        assert_equal(t1, t2)
        assert_equal(t2, t3)



from __future__ import division, absolute_import, print_function

from numpy.testing import *
import numpy as np
from numpy import ( array, ones, r_, mgrid, unravel_index, zeros, where,
                    ndenumerate, fill_diagonal, diag_indices,
                    diag_indices_from, s_, index_exp, ndindex )

class TestRavelUnravelIndex(TestCase):
    def test_basic(self):
        assert_equal(np.unravel_index(2, (2, 2)), (1, 0))
        assert_equal(np.ravel_multi_index((1, 0), (2, 2)), 2)
        assert_equal(np.unravel_index(254, (17, 94)), (2, 66))
        assert_equal(np.ravel_multi_index((2, 66), (17, 94)), 254)
        assert_raises(ValueError, np.unravel_index, -1, (2, 2))
        assert_raises(TypeError, np.unravel_index, 0.5, (2, 2))
        assert_raises(ValueError, np.unravel_index, 4, (2, 2))
        assert_raises(ValueError, np.ravel_multi_index, (-3, 1), (2, 2))
        assert_raises(ValueError, np.ravel_multi_index, (2, 1), (2, 2))
        assert_raises(ValueError, np.ravel_multi_index, (0, -3), (2, 2))
        assert_raises(ValueError, np.ravel_multi_index, (0, 2), (2, 2))
        assert_raises(TypeError, np.ravel_multi_index, (0.1, 0.), (2, 2))

        assert_equal(np.unravel_index((2*3 + 1)*6 + 4, (4, 3, 6)), [2, 1, 4])
        assert_equal(np.ravel_multi_index([2, 1, 4], (4, 3, 6)), (2*3 + 1)*6 + 4)

        arr = np.array([[3, 6, 6], [4, 5, 1]])
        assert_equal(np.ravel_multi_index(arr, (7, 6)), [22, 41, 37])
        assert_equal(np.ravel_multi_index(arr, (7, 6), order='F'), [31, 41, 13])
        assert_equal(np.ravel_multi_index(arr, (4, 6), mode='clip'), [22, 23, 19])
        assert_equal(np.ravel_multi_index(arr, (4, 4), mode=('clip', 'wrap')),
                        [12, 13, 13])
        assert_equal(np.ravel_multi_index((3, 1, 4, 1), (6, 7, 8, 9)), 1621)

        assert_equal(np.unravel_index(np.array([22, 41, 37]), (7, 6)),
                        [[3, 6, 6], [4, 5, 1]])
        assert_equal(np.unravel_index(np.array([31, 41, 13]), (7, 6), order='F'),
                        [[3, 6, 6], [4, 5, 1]])
        assert_equal(np.unravel_index(1621, (6, 7, 8, 9)), [3, 1, 4, 1])

    def test_dtypes(self):
        # Test with different data types
        for dtype in [np.int16, np.uint16, np.int32,
                      np.uint32, np.int64, np.uint64]:
            coords = np.array([[1, 0, 1, 2, 3, 4], [1, 6, 1, 3, 2, 0]], dtype=dtype)
            shape = (5, 8)
            uncoords = 8*coords[0]+coords[1]
            assert_equal(np.ravel_multi_index(coords, shape), uncoords)
            assert_equal(coords, np.unravel_index(uncoords, shape))
            uncoords = coords[0]+5*coords[1]
            assert_equal(np.ravel_multi_index(coords, shape, order='F'), uncoords)
            assert_equal(coords, np.unravel_index(uncoords, shape, order='F'))

            coords = np.array([[1, 0, 1, 2, 3, 4], [1, 6, 1, 3, 2, 0], [1, 3, 1, 0, 9, 5]],
                              dtype=dtype)
            shape = (5, 8, 10)
            uncoords = 10*(8*coords[0]+coords[1])+coords[2]
            assert_equal(np.ravel_multi_index(coords, shape), uncoords)
            assert_equal(coords, np.unravel_index(uncoords, shape))
            uncoords = coords[0]+5*(coords[1]+8*coords[2])
            assert_equal(np.ravel_multi_index(coords, shape, order='F'), uncoords)
            assert_equal(coords, np.unravel_index(uncoords, shape, order='F'))

    def test_clipmodes(self):
        # Test clipmodes
        assert_equal(np.ravel_multi_index([5, 1, -1, 2], (4, 3, 7, 12), mode='wrap'),
                     np.ravel_multi_index([1, 1, 6, 2], (4, 3, 7, 12)))
        assert_equal(np.ravel_multi_index([5, 1, -1, 2], (4, 3, 7, 12),
                                 mode=('wrap', 'raise', 'clip', 'raise')),
                     np.ravel_multi_index([1, 1, 0, 2], (4, 3, 7, 12)))
        assert_raises(ValueError, np.ravel_multi_index, [5, 1, -1, 2], (4, 3, 7, 12))

class TestGrid(TestCase):
    def test_basic(self):
        a = mgrid[-1:1:10j]
        b = mgrid[-1:1:0.1]
        assert_(a.shape == (10,))
        assert_(b.shape == (20,))
        assert_(a[0] == -1)
        assert_almost_equal(a[-1], 1)
        assert_(b[0] == -1)
        assert_almost_equal(b[1]-b[0], 0.1, 11)
        assert_almost_equal(b[-1], b[0]+19*0.1, 11)
        assert_almost_equal(a[1]-a[0], 2.0/9.0, 11)

    def test_linspace_equivalence(self):
        y, st = np.linspace(2, 10, retstep=1)
        assert_almost_equal(st, 8/49.0)
        assert_array_almost_equal(y, mgrid[2:10:50j], 13)

    def test_nd(self):
        c = mgrid[-1:1:10j, -2:2:10j]
        d = mgrid[-1:1:0.1, -2:2:0.2]
        assert_(c.shape == (2, 10, 10))
        assert_(d.shape == (2, 20, 20))
        assert_array_equal(c[0][0,:], -ones(10, 'd'))
        assert_array_equal(c[1][:, 0], -2*ones(10, 'd'))
        assert_array_almost_equal(c[0][-1,:], ones(10, 'd'), 11)
        assert_array_almost_equal(c[1][:, -1], 2*ones(10, 'd'), 11)
        assert_array_almost_equal(d[0, 1,:]-d[0, 0,:], 0.1*ones(20, 'd'), 11)
        assert_array_almost_equal(d[1,:, 1]-d[1,:, 0], 0.2*ones(20, 'd'), 11)


class TestConcatenator(TestCase):
    def test_1d(self):
        assert_array_equal(r_[1, 2, 3, 4, 5, 6], array([1, 2, 3, 4, 5, 6]))
        b = ones(5)
        c = r_[b, 0, 0, b]
        assert_array_equal(c, [1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1])

    def test_mixed_type(self):
        g = r_[10.1, 1:10]
        assert_(g.dtype == 'f8')

    def test_more_mixed_type(self):
        g = r_[-10.1, array([1]), array([2, 3, 4]), 10.0]
        assert_(g.dtype == 'f8')

    def test_2d(self):
        b = rand(5, 5)
        c = rand(5, 5)
        d = r_['1', b, c]  # append columns
        assert_(d.shape == (5, 10))
        assert_array_equal(d[:, :5], b)
        assert_array_equal(d[:, 5:], c)
        d = r_[b, c]
        assert_(d.shape == (10, 5))
        assert_array_equal(d[:5,:], b)
        assert_array_equal(d[5:,:], c)


class TestNdenumerate(TestCase):
    def test_basic(self):
        a = array([[1, 2], [3, 4]])
        assert_equal(list(ndenumerate(a)),
                     [((0, 0), 1), ((0, 1), 2), ((1, 0), 3), ((1, 1), 4)])


class TestIndexExpression(TestCase):
    def test_regression_1(self):
        # ticket #1196
        a = np.arange(2)
        assert_equal(a[:-1], a[s_[:-1]])
        assert_equal(a[:-1], a[index_exp[:-1]])

    def test_simple_1(self):
        a = np.random.rand(4, 5, 6)

        assert_equal(a[:, :3, [1, 2]], a[index_exp[:, :3, [1, 2]]])
        assert_equal(a[:, :3, [1, 2]], a[s_[:, :3, [1, 2]]])

def test_c_():
    a = np.c_[np.array([[1, 2, 3]]), 0, 0, np.array([[4, 5, 6]])]
    assert_equal(a, [[1, 2, 3, 0, 0, 4, 5, 6]])

def test_fill_diagonal():
    a = zeros((3, 3), int)
    fill_diagonal(a, 5)
    yield (assert_array_equal, a,
           array([[5, 0, 0],
                  [0, 5, 0],
                  [0, 0, 5]]))

    #Test tall matrix
    a = zeros((10, 3), int)
    fill_diagonal(a, 5)
    yield (assert_array_equal, a,
           array([[5, 0, 0],
                  [0, 5, 0],
                  [0, 0, 5],
                  [0, 0, 0],
                  [0, 0, 0],
                  [0, 0, 0],
                  [0, 0, 0],
                  [0, 0, 0],
                  [0, 0, 0],
                  [0, 0, 0]]))

    #Test tall matrix wrap
    a = zeros((10, 3), int)
    fill_diagonal(a, 5, True)
    yield (assert_array_equal, a,
           array([[5, 0, 0],
                  [0, 5, 0],
                  [0, 0, 5],
                  [0, 0, 0],
                  [5, 0, 0],
                  [0, 5, 0],
                  [0, 0, 5],
                  [0, 0, 0],
                  [5, 0, 0],
                  [0, 5, 0]]))

    #Test wide matrix
    a = zeros((3, 10), int)
    fill_diagonal(a, 5)
    yield (assert_array_equal, a,
           array([[5, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                  [0, 5, 0, 0, 0, 0, 0, 0, 0, 0],
                  [0, 0, 5, 0, 0, 0, 0, 0, 0, 0]]))

    # The same function can operate on a 4-d array:
    a = zeros((3, 3, 3, 3), int)
    fill_diagonal(a, 4)
    i = array([0, 1, 2])
    yield (assert_equal, where(a != 0), (i, i, i, i))


def test_diag_indices():
    di = diag_indices(4)
    a = array([[1, 2, 3, 4],
               [5, 6, 7, 8],
               [9, 10, 11, 12],
               [13, 14, 15, 16]])
    a[di] = 100
    yield (assert_array_equal, a,
           array([[100,   2,   3,   4],
                  [  5, 100,   7,   8],
                  [  9,  10, 100,  12],
                  [ 13,  14,  15, 100]]))

    # Now, we create indices to manipulate a 3-d array:
    d3 = diag_indices(2, 3)

    # And use it to set the diagonal of a zeros array to 1:
    a = zeros((2, 2, 2), int)
    a[d3] = 1
    yield (assert_array_equal, a,
           array([[[1, 0],
                   [0, 0]],

                  [[0, 0],
                   [0, 1]]]) )

def test_diag_indices_from():
    x = np.random.random((4, 4))
    r, c = diag_indices_from(x)
    assert_array_equal(r, np.arange(4))
    assert_array_equal(c, np.arange(4))


def test_ndindex():
    x = list(np.ndindex(1, 2, 3))
    expected = [ix for ix, e in np.ndenumerate(np.zeros((1, 2, 3)))]
    assert_array_equal(x, expected)

    x = list(np.ndindex((1, 2, 3)))
    assert_array_equal(x, expected)

    # Test use of scalars and tuples
    x = list(np.ndindex((3,)))
    assert_array_equal(x, list(np.ndindex(3)))

    # Make sure size argument is optional
    x = list(np.ndindex())
    assert_equal(x, [()])

    x = list(np.ndindex(()))
    assert_equal(x, [()])

    # Make sure 0-sized ndindex works correctly
    x = list(np.ndindex(*[0]))
    assert_equal(x, [])


if __name__ == "__main__":
    run_module_suite()



#!/usr/bin/env python
'''
Created on 22-04-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''
import os
import sys
import threading
import time

from aqopa.model.parser import ModelParserException,\
    MetricsParserException, ConfigurationParserException
from aqopa.app import Builder, ConsoleInterpreter
from aqopa.simulator import EnvironmentDefinitionException
from aqopa.module import timeanalysis, energyanalysis, reputation


class ProgressThread(threading.Thread):

    def __init__(self, f, interpreter, *args, **kwargs):
        super(ProgressThread, self).__init__(*args, **kwargs)
        self.file = f
        self.interpreter = interpreter
        self.signs = '|/-\\'
        self.sign_no = 0
        
    def get_progress(self):
        all_progress = 0.0
        sum_progress = 0.0
        for s in self.interpreter.simulators:
            all_progress += 1
            sum_progress += s.context.get_progress()
        progress = 0
        if all_progress > 0:
            progress = sum_progress / all_progress
        return progress
            
    def run(self):
        
        progress = self.get_progress()
        while progress < 1 and not self.interpreter.is_finished():
            self.print_progressbar(progress)
            time.sleep(0.2)
            progress = self.get_progress()
        self.print_progressbar(progress)
        self.file.write("\n")

        
    def print_progressbar(self, progress):
            """
            Prints the formatted progressbar showing the progress of simulation. 
            """
            self.sign_no += 1
            sign = self.signs[self.sign_no % len(self.signs)]
            
            percentage = str(int(round(progress*100))) + '%'
            percentage = (' ' * (5-len(percentage))) + percentage
            
            bar = ('#' * int(round(progress*20))) + (' ' * (20 - int(round(progress*20))))
            
            self.file.write("\r%c[%s] %s" % (sign, bar, percentage))
            self.file.flush()
        

def run(qopml_model, qopml_metrics, qopml_configuration, 
         save_states = False, debug = False, show_progressbar = False):

    # Prepare all available modules

    time_module = timeanalysis.Module()
    energy_module = energyanalysis.Module(time_module)
    reputation_module = reputation.Module()

    available_modules = [time_module, energy_module, reputation_module]

    ############### DEBUG ###############    
    if debug:
        builder = Builder()
        store = builder.build_store()
        parser = builder.build_model_parser(store, available_modules)
        parser.lexer.input(qopml_model)
        while True:
            print  parser.lexer.current_state()
            tok = parser.lexer.token()
            if not tok:
                break
            print tok
            print ""
        print 'Errors: ' + str(parser.get_syntax_errors())
        print ""
        print ""

        return
    
    #####################################
    interpreter = ConsoleInterpreter()
    try:

        interpreter.set_qopml_model(qopml_model)
        interpreter.set_qopml_metrics(qopml_metrics)
        interpreter.set_qopml_config(qopml_configuration)

        interpreter.register_qopml_module(time_module)
        interpreter.register_qopml_module(energy_module)
        interpreter.register_qopml_module(reputation_module)
        
        interpreter.parse(available_modules)
        interpreter.prepare()
        
        if save_states:
            for simulator in interpreter.simulators: 
                interpreter.save_states_to_file(simulator)
        
        if show_progressbar:
            progressbar_thread = ProgressThread(sys.stdout, interpreter)
            progressbar_thread.start()
        
        interpreter.run()

    except EnvironmentDefinitionException, e:
        sys.stderr.write('Error on creating environment: %s\n' % e)
        if len(e.errors) > 0:
            sys.stderr.write('Errors:\n')
            sys.stderr.write('\n'.join(e.errors))
            sys.stderr.write('\n')
    except ModelParserException, e:
        sys.stderr.write('Model parsing error: %s\n' % e)
        if len(e.syntax_errors):
            sys.stderr.write('Syntax errors:\n')
            sys.stderr.write('\n'.join(e.syntax_errors))
            sys.stderr.write('\n')
    except MetricsParserException, e:
        sys.stderr.write('Metrics parsing error: %s\n' % e)
        if len(e.syntax_errors):
            sys.stderr.write('Syntax errors:\n')
            sys.stderr.write('\n'.join(e.syntax_errors))
            sys.stderr.write('\n')
    except ConfigurationParserException, e:
        sys.stderr.write('Configuration parsing error: %s\n' % e)
        if len(e.syntax_errors):
            sys.stderr.write('Syntax errors:\n')
            sys.stderr.write('\n'.join(e.syntax_errors))
            sys.stderr.write('\n')
        



"""
Define a simple format for saving numpy arrays to disk with the full
information about them.

The ``.npy`` format is the standard binary file format in NumPy for
persisting a *single* arbitrary NumPy array on disk. The format stores all
of the shape and dtype information necessary to reconstruct the array
correctly even on another machine with a different architecture.
The format is designed to be as simple as possible while achieving
its limited goals.

The ``.npz`` format is the standard format for persisting *multiple* NumPy
arrays on disk. A ``.npz`` file is a zip file containing multiple ``.npy``
files, one for each array.

Capabilities
------------

- Can represent all NumPy arrays including nested record arrays and
  object arrays.

- Represents the data in its native binary form.

- Supports Fortran-contiguous arrays directly.

- Stores all of the necessary information to reconstruct the array
  including shape and dtype on a machine of a different
  architecture.  Both little-endian and big-endian arrays are
  supported, and a file with little-endian numbers will yield
  a little-endian array on any machine reading the file. The
  types are described in terms of their actual sizes. For example,
  if a machine with a 64-bit C "long int" writes out an array with
  "long ints", a reading machine with 32-bit C "long ints" will yield
  an array with 64-bit integers.

- Is straightforward to reverse engineer. Datasets often live longer than
  the programs that created them. A competent developer should be
  able to create a solution in his preferred programming language to
  read most ``.npy`` files that he has been given without much
  documentation.

- Allows memory-mapping of the data. See `open_memmep`.

- Can be read from a filelike stream object instead of an actual file.

- Stores object arrays, i.e. arrays containing elements that are arbitrary
  Python objects. Files with object arrays are not to be mmapable, but
  can be read and written to disk.

Limitations
-----------

- Arbitrary subclasses of numpy.ndarray are not completely preserved.
  Subclasses will be accepted for writing, but only the array data will
  be written out. A regular numpy.ndarray object will be created
  upon reading the file.

.. warning::

  Due to limitations in the interpretation of structured dtypes, dtypes
  with fields with empty names will have the names replaced by 'f0', 'f1',
  etc. Such arrays will not round-trip through the format entirely
  accurately. The data is intact; only the field names will differ. We are
  working on a fix for this. This fix will not require a change in the
  file format. The arrays with such structures can still be saved and
  restored, and the correct dtype may be restored by using the
  ``loadedarray.view(correct_dtype)`` method.

File extensions
---------------

We recommend using the ``.npy`` and ``.npz`` extensions for files saved
in this format. This is by no means a requirement; applications may wish
to use these file formats but use an extension specific to the
application. In the absence of an obvious alternative, however,
we suggest using ``.npy`` and ``.npz``.

Version numbering
-----------------

The version numbering of these formats is independent of NumPy version
numbering. If the format is upgraded, the code in `numpy.io` will still
be able to read and write Version 1.0 files.

Format Version 1.0
------------------

The first 6 bytes are a magic string: exactly ``\\x93NUMPY``.

The next 1 byte is an unsigned byte: the major version number of the file
format, e.g. ``\\x01``.

The next 1 byte is an unsigned byte: the minor version number of the file
format, e.g. ``\\x00``. Note: the version of the file format is not tied
to the version of the numpy package.

The next 2 bytes form a little-endian unsigned short int: the length of
the header data HEADER_LEN.

The next HEADER_LEN bytes form the header data describing the array's
format. It is an ASCII string which contains a Python literal expression
of a dictionary. It is terminated by a newline (``\\n``) and padded with
spaces (``\\x20``) to make the total length of
``magic string + 4 + HEADER_LEN`` be evenly divisible by 16 for alignment
purposes.

The dictionary contains three keys:

    "descr" : dtype.descr
      An object that can be passed as an argument to the `numpy.dtype`
      constructor to create the array's dtype.
    "fortran_order" : bool
      Whether the array data is Fortran-contiguous or not. Since
      Fortran-contiguous arrays are a common form of non-C-contiguity,
      we allow them to be written directly to disk for efficiency.
    "shape" : tuple of int
      The shape of the array.

For repeatability and readability, the dictionary keys are sorted in
alphabetic order. This is for convenience only. A writer SHOULD implement
this if possible. A reader MUST NOT depend on this.

Following the header comes the array data. If the dtype contains Python
objects (i.e. ``dtype.hasobject is True``), then the data is a Python
pickle of the array. Otherwise the data is the contiguous (either C-
or Fortran-, depending on ``fortran_order``) bytes of the array.
Consumers can figure out the number of bytes by multiplying the number
of elements given by the shape (noting that ``shape=()`` means there is
1 element) by ``dtype.itemsize``.

Notes
-----
The ``.npy`` format, including reasons for creating it and a comparison of
alternatives, is described fully in the "npy-format" NEP.

"""
from __future__ import division, absolute_import, print_function

import numpy
import sys
from numpy.lib.utils import safe_eval
from numpy.compat import asbytes, isfileobj, long, basestring

if sys.version_info[0] >= 3:
    import pickle
else:
    import cPickle as pickle

MAGIC_PREFIX = asbytes('\x93NUMPY')
MAGIC_LEN = len(MAGIC_PREFIX) + 2
BUFFER_SIZE = 2 ** 18 #size of buffer for reading npz files in bytes

def magic(major, minor):
    """ Return the magic string for the given file format version.

    Parameters
    ----------
    major : int in [0, 255]
    minor : int in [0, 255]

    Returns
    -------
    magic : str

    Raises
    ------
    ValueError if the version cannot be formatted.
    """
    if major < 0 or major > 255:
        raise ValueError("major version must be 0 <= major < 256")
    if minor < 0 or minor > 255:
        raise ValueError("minor version must be 0 <= minor < 256")
    if sys.version_info[0] < 3:
        return MAGIC_PREFIX + chr(major) + chr(minor)
    else:
        return MAGIC_PREFIX + bytes([major, minor])

def read_magic(fp):
    """ Read the magic string to get the version of the file format.

    Parameters
    ----------
    fp : filelike object

    Returns
    -------
    major : int
    minor : int
    """
    magic_str = fp.read(MAGIC_LEN)
    if len(magic_str) != MAGIC_LEN:
        msg = "could not read %d characters for the magic string; got %r"
        raise ValueError(msg % (MAGIC_LEN, magic_str))
    if magic_str[:-2] != MAGIC_PREFIX:
        msg = "the magic string is not correct; expected %r, got %r"
        raise ValueError(msg % (MAGIC_PREFIX, magic_str[:-2]))
    if sys.version_info[0] < 3:
        major, minor = map(ord, magic_str[-2:])
    else:
        major, minor = magic_str[-2:]
    return major, minor

def dtype_to_descr(dtype):
    """
    Get a serializable descriptor from the dtype.

    The .descr attribute of a dtype object cannot be round-tripped through
    the dtype() constructor. Simple types, like dtype('float32'), have
    a descr which looks like a record array with one field with '' as
    a name. The dtype() constructor interprets this as a request to give
    a default name.  Instead, we construct descriptor that can be passed to
    dtype().

    Parameters
    ----------
    dtype : dtype
        The dtype of the array that will be written to disk.

    Returns
    -------
    descr : object
        An object that can be passed to `numpy.dtype()` in order to
        replicate the input dtype.

    """
    if dtype.names is not None:
        # This is a record array. The .descr is fine.
        # XXX: parts of the record array with an empty name, like padding bytes,
        # still get fiddled with. This needs to be fixed in the C implementation
        # of dtype().
        return dtype.descr
    else:
        return dtype.str

def header_data_from_array_1_0(array):
    """ Get the dictionary of header metadata from a numpy.ndarray.

    Parameters
    ----------
    array : numpy.ndarray

    Returns
    -------
    d : dict
        This has the appropriate entries for writing its string representation
        to the header of the file.
    """
    d = {}
    d['shape'] = array.shape
    if array.flags.c_contiguous:
        d['fortran_order'] = False
    elif array.flags.f_contiguous:
        d['fortran_order'] = True
    else:
        # Totally non-contiguous data. We will have to make it C-contiguous
        # before writing. Note that we need to test for C_CONTIGUOUS first
        # because a 1-D array is both C_CONTIGUOUS and F_CONTIGUOUS.
        d['fortran_order'] = False

    d['descr'] = dtype_to_descr(array.dtype)
    return d

def write_array_header_1_0(fp, d):
    """ Write the header for an array using the 1.0 format.

    Parameters
    ----------
    fp : filelike object
    d : dict
        This has the appropriate entries for writing its string representation
        to the header of the file.
    """
    import struct
    header = ["{"]
    for key, value in sorted(d.items()):
        # Need to use repr here, since we eval these when reading
        header.append("'%s': %s, " % (key, repr(value)))
    header.append("}")
    header = "".join(header)
    # Pad the header with spaces and a final newline such that the magic
    # string, the header-length short and the header are aligned on a 16-byte
    # boundary.  Hopefully, some system, possibly memory-mapping, can take
    # advantage of our premature optimization.
    current_header_len = MAGIC_LEN + 2 + len(header) + 1  # 1 for the newline
    topad = 16 - (current_header_len % 16)
    header = asbytes(header + ' '*topad + '\n')
    if len(header) >= (256*256):
        raise ValueError("header does not fit inside %s bytes" % (256*256))
    header_len_str = struct.pack('<H', len(header))
    fp.write(header_len_str)
    fp.write(header)

def read_array_header_1_0(fp):
    """
    Read an array header from a filelike object using the 1.0 file format
    version.

    This will leave the file object located just after the header.

    Parameters
    ----------
    fp : filelike object
        A file object or something with a `.read()` method like a file.

    Returns
    -------
    shape : tuple of int
        The shape of the array.
    fortran_order : bool
        The array data will be written out directly if it is either C-contiguous
        or Fortran-contiguous. Otherwise, it will be made contiguous before
        writing it out.
    dtype : dtype
        The dtype of the file's data.

    Raises
    ------
    ValueError
        If the data is invalid.

    """
    # Read an unsigned, little-endian short int which has the length of the
    # header.
    import struct
    hlength_str = fp.read(2)
    if len(hlength_str) != 2:
        msg = "EOF at %s before reading array header length"
        raise ValueError(msg % fp.tell())
    header_length = struct.unpack('<H', hlength_str)[0]
    header = fp.read(header_length)
    if len(header) != header_length:
        raise ValueError("EOF at %s before reading array header" % fp.tell())

    # The header is a pretty-printed string representation of a literal Python
    # dictionary with trailing newlines padded to a 16-byte boundary. The keys
    # are strings.
    #   "shape" : tuple of int
    #   "fortran_order" : bool
    #   "descr" : dtype.descr
    try:
        d = safe_eval(header)
    except SyntaxError as e:
        msg = "Cannot parse header: %r\nException: %r"
        raise ValueError(msg % (header, e))
    if not isinstance(d, dict):
        msg = "Header is not a dictionary: %r"
        raise ValueError(msg % d)
    keys = sorted(d.keys())
    if keys != ['descr', 'fortran_order', 'shape']:
        msg = "Header does not contain the correct keys: %r"
        raise ValueError(msg % (keys,))

    # Sanity-check the values.
    if (not isinstance(d['shape'], tuple) or
        not numpy.all([isinstance(x, (int, long)) for x in d['shape']])):
        msg = "shape is not valid: %r"
        raise ValueError(msg % (d['shape'],))
    if not isinstance(d['fortran_order'], bool):
        msg = "fortran_order is not a valid bool: %r"
        raise ValueError(msg % (d['fortran_order'],))
    try:
        dtype = numpy.dtype(d['descr'])
    except TypeError as e:
        msg = "descr is not a valid dtype descriptor: %r"
        raise ValueError(msg % (d['descr'],))

    return d['shape'], d['fortran_order'], dtype

def write_array(fp, array, version=(1, 0)):
    """
    Write an array to an NPY file, including a header.

    If the array is neither C-contiguous nor Fortran-contiguous AND the
    file_like object is not a real file object, this function will have to
    copy data in memory.

    Parameters
    ----------
    fp : file_like object
        An open, writable file object, or similar object with a ``.write()``
        method.
    array : ndarray
        The array to write to disk.
    version : (int, int), optional
        The version number of the format.  Default: (1, 0)

    Raises
    ------
    ValueError
        If the array cannot be persisted.
    Various other errors
        If the array contains Python objects as part of its dtype, the
        process of pickling them may raise various errors if the objects
        are not picklable.

    """
    if version != (1, 0):
        msg = "we only support format version (1,0), not %s"
        raise ValueError(msg % (version,))
    fp.write(magic(*version))
    write_array_header_1_0(fp, header_data_from_array_1_0(array))
    if array.dtype.hasobject:
        # We contain Python objects so we cannot write out the data directly.
        # Instead, we will pickle it out with version 2 of the pickle protocol.
        pickle.dump(array, fp, protocol=2)
    elif array.flags.f_contiguous and not array.flags.c_contiguous:
        if isfileobj(fp):
            array.T.tofile(fp)
        else:
            fp.write(array.T.tostring('C'))
    else:
        if isfileobj(fp):
            array.tofile(fp)
        else:
            # XXX: We could probably chunk this using something like
            # arrayterator.
            fp.write(array.tostring('C'))

def read_array(fp):
    """
    Read an array from an NPY file.

    Parameters
    ----------
    fp : file_like object
        If this is not a real file object, then this may take extra memory
        and time.

    Returns
    -------
    array : ndarray
        The array from the data on disk.

    Raises
    ------
    ValueError
        If the data is invalid.

    """
    version = read_magic(fp)
    if version != (1, 0):
        msg = "only support version (1,0) of file format, not %r"
        raise ValueError(msg % (version,))
    shape, fortran_order, dtype = read_array_header_1_0(fp)
    if len(shape) == 0:
        count = 1
    else:
        count = numpy.multiply.reduce(shape)

    # Now read the actual data.
    if dtype.hasobject:
        # The array contained Python objects. We need to unpickle the data.
        array = pickle.load(fp)
    else:
        if isfileobj(fp):
            # We can use the fast fromfile() function.
            array = numpy.fromfile(fp, dtype=dtype, count=count)
        else:
            # This is not a real file. We have to read it the memory-intensive
            # way.
            # crc32 module fails on reads greater than 2 ** 32 bytes, breaking
            # large reads from gzip streams. Chunk reads to BUFFER_SIZE bytes to
            # avoid issue and reduce memory overhead of the read. In
            # non-chunked case count < max_read_count, so only one read is
            # performed.

            max_read_count = BUFFER_SIZE // dtype.itemsize

            array = numpy.empty(count, dtype=dtype)

            for i in range(0, count, max_read_count):
                read_count = min(max_read_count, count - i)

                data = fp.read(int(read_count * dtype.itemsize))
                array[i:i+read_count] = numpy.frombuffer(data, dtype=dtype,
                                                         count=read_count)

        if fortran_order:
            array.shape = shape[::-1]
            array = array.transpose()
        else:
            array.shape = shape

    return array


def open_memmap(filename, mode='r+', dtype=None, shape=None,
                fortran_order=False, version=(1, 0)):
    """
    Open a .npy file as a memory-mapped array.

    This may be used to read an existing file or create a new one.

    Parameters
    ----------
    filename : str
        The name of the file on disk.  This may *not* be a file-like
        object.
    mode : str, optional
        The mode in which to open the file; the default is 'r+'.  In
        addition to the standard file modes, 'c' is also accepted to
        mean "copy on write."  See `memmap` for the available mode strings.
    dtype : data-type, optional
        The data type of the array if we are creating a new file in "write"
        mode, if not, `dtype` is ignored.  The default value is None,
        which results in a data-type of `float64`.
    shape : tuple of int
        The shape of the array if we are creating a new file in "write"
        mode, in which case this parameter is required.  Otherwise, this
        parameter is ignored and is thus optional.
    fortran_order : bool, optional
        Whether the array should be Fortran-contiguous (True) or
        C-contiguous (False, the default) if we are creating a new file
        in "write" mode.
    version : tuple of int (major, minor)
        If the mode is a "write" mode, then this is the version of the file
        format used to create the file.  Default: (1,0)

    Returns
    -------
    marray : memmap
        The memory-mapped array.

    Raises
    ------
    ValueError
        If the data or the mode is invalid.
    IOError
        If the file is not found or cannot be opened correctly.

    See Also
    --------
    memmap

    """
    if not isinstance(filename, basestring):
        raise ValueError("Filename must be a string.  Memmap cannot use" \
                         " existing file handles.")

    if 'w' in mode:
        # We are creating the file, not reading it.
        # Check if we ought to create the file.
        if version != (1, 0):
            msg = "only support version (1,0) of file format, not %r"
            raise ValueError(msg % (version,))
        # Ensure that the given dtype is an authentic dtype object rather than
        # just something that can be interpreted as a dtype object.
        dtype = numpy.dtype(dtype)
        if dtype.hasobject:
            msg = "Array can't be memory-mapped: Python objects in dtype."
            raise ValueError(msg)
        d = dict(
            descr=dtype_to_descr(dtype),
            fortran_order=fortran_order,
            shape=shape,
        )
        # If we got here, then it should be safe to create the file.
        fp = open(filename, mode+'b')
        try:
            fp.write(magic(*version))
            write_array_header_1_0(fp, d)
            offset = fp.tell()
        finally:
            fp.close()
    else:
        # Read the header of the file first.
        fp = open(filename, 'rb')
        try:
            version = read_magic(fp)
            if version != (1, 0):
                msg = "only support version (1,0) of file format, not %r"
                raise ValueError(msg % (version,))
            shape, fortran_order, dtype = read_array_header_1_0(fp)
            if dtype.hasobject:
                msg = "Array can't be memory-mapped: Python objects in dtype."
                raise ValueError(msg)
            offset = fp.tell()
        finally:
            fp.close()

    if fortran_order:
        order = 'F'
    else:
        order = 'C'

    # We need to change a write-only mode to a read-write mode since we've
    # already written data to the file.
    if mode == 'w+':
        mode = 'r+'

    marray = numpy.memmap(filename, dtype=dtype, shape=shape, order=order,
        mode=mode, offset=offset)

    return marray



'''
Created on 07-05-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''
from aqopa.simulator.state import HOOK_TYPE_PRE_HOST_LIST_EXECUTION,\
    HookExecutor, HOOK_TYPE_PRE_INSTRUCTION_EXECUTION,\
    HOOK_TYPE_POST_INSTRUCTION_EXECUTION, HOOK_TYPE_SIMULATION_FINISHED
    
from aqopa.simulator.error import RuntimeException,\
    EnvironmentDefinitionException, InfiniteLoopException

class Simulator():
    """
    Interpreter's Model Simulator
    """
    
    def __init__(self, context):
        self.context = context  # Simulation context (keeps information about current state)
        self._ready = False
        self._hooks = {}
        self._modules = []
        
        self._executor = None
        self._before_instruction_executor = HookExecutor()
        self._after_instruction_executor = HookExecutor()
        
        self._first_loop = True
        self._infinite_loop_error = False
        
    def _execute_hook(self, hook_type):
        """
        Execute all hooks of given type.
        Pre instruction and post instruction hooks 
        cannot be executet manually.
        """
        
        if hook_type not in self._hooks:
            return
        
        if hook_type in [HOOK_TYPE_PRE_INSTRUCTION_EXECUTION, HOOK_TYPE_POST_INSTRUCTION_EXECUTION]:
            raise RuntimeException("Cannot execute pre instruction and post instruction hooks manually.")
        
        for h in self._hooks[hook_type]:
            h.execute(self.context)
    
    def _install_modules(self):
        """
        Method installs registered modules.
        """
        if self._ready:
            raise EnvironmentDefinitionException('Cannot install modules in prepared simulation, they were already installed.')
        
        for m in self._modules:
            m.install(self)
    
    def _internal_goto_next_state(self):
        """
        Internally goes to next state.
        The difference is that it does not check 
        whether simulation is finished.
        """
        if self.context.has_epoch_ended():
            self._execute_hook(HOOK_TYPE_PRE_HOST_LIST_EXECUTION)
            
            if self._first_loop:
                self._first_loop = False
            else:
                # If nothing has changed in this next state generation loop
                if not self.context.any_host_changed():
                    # Throw runtime error if any channel has message to be binded with receiver
                    # or if any channel has dropped a message
                    for ch in self.context.channels_manager.channels:
                        if len(ch.get_queue_of_sending_hosts(1)) > 0 or ch.get_number_of_dropped_messages() > 0:
                            raise InfiniteLoopException()
                        
                    for h in self.context.hosts:
                        if not h.finished():
                            h.finish_failed(u'Infinite loop occured on instruction: %s' % 
                                            unicode(h.get_current_instructions_context().get_current_instruction()))
                            
                self.context.mark_all_hosts_unchanged()

        self.context.get_current_host().touch()
        self._executor.execute_instruction(self.context)
        self.context.goto_next_host()
    
    # Public
    
    def get_executor(self):
        """ executor getter """
        return self._executor
    
    def set_executor(self, executor):
        """ executor setter """
        self._executor = executor
        
    def infinite_loop_occured(self):
        """ Returns True if infinite loop occured """
        return self._infinite_loop_error
    
    def register_module(self, module):
        """
        Register module for simulation.
        """
        if self._ready:
            raise EnvironmentDefinitionException('Cannot register module in prepared simulation.')
        self._modules.append(module)
        return self
    
    def prepare(self):
        """
        Prepare simulator to start.
        """
        self._install_modules()
        self._first_loop = True
        self._ready = True
        return self
        
    def register_hook(self, hook_type, hook):
        """
        Registers new hook of particular type
        """
        if hook_type not in self._hooks:
            self._hooks[hook_type] = []
            
        if hook in self._hooks[hook_type]:
            raise EnvironmentDefinitionException(u"Hook '%s' is already registered." % unicode(hook))
        self._hooks[hook_type].append(hook)

        if hook_type == HOOK_TYPE_PRE_INSTRUCTION_EXECUTION:
            self._before_instruction_executor.add_hook(hook)
            
        if hook_type == HOOK_TYPE_POST_INSTRUCTION_EXECUTION:
            self._after_instruction_executor.add_hook(hook)
        
        return self
    
    def is_ready_to_run(self):
        """
        Returns True if simulator is ready to run
        """
        return self._ready
    
    def is_simulation_finished(self):
        """
        Returns True if simulation has ended.
        Simulation can end with success or error (eg. infinite loop occured).
        """
        return self.is_ready_to_run() and (self.context.all_hosts_finished() or self.infinite_loop_occured())
    
    def run(self):
        """
        Runs whole simulation process.
        """
        if not self.is_ready_to_run():
            raise EnvironmentDefinitionException("Simulation is not yet ready to run.")

        self._executor.prepend_instruction_executor(self._before_instruction_executor)
        self._executor.append_instruction_executor(self._after_instruction_executor)
        
        while not self.is_simulation_finished():
            try: 
                self._internal_goto_next_state()
            except InfiniteLoopException:
                self._infinite_loop_error = True

        self._execute_hook(HOOK_TYPE_SIMULATION_FINISHED)
        



"""
This module adds the default axis argument to code which did not specify it
for the functions where the default was changed in NumPy.

The functions changed are

add -1  ( all second argument)
======
nansum
nanmax
nanmin
nanargmax
nanargmin
argmax
argmin
compress 3


add 0
======
take     3
repeat   3
sum         # might cause problems with builtin.
product
sometrue
alltrue
cumsum
cumproduct
average
ptp
cumprod
prod
std
mean

"""
from __future__ import division, absolute_import, print_function

__all__ = ['convertfile', 'convertall', 'converttree']

import sys
import os
import re
import glob


_args3 = ['compress', 'take', 'repeat']
_funcm1 = ['nansum', 'nanmax', 'nanmin', 'nanargmax', 'nanargmin',
           'argmax', 'argmin', 'compress']
_func0 = ['take', 'repeat', 'sum', 'product', 'sometrue', 'alltrue',
          'cumsum', 'cumproduct', 'average', 'ptp', 'cumprod', 'prod',
          'std', 'mean']

_all = _func0 + _funcm1
func_re = {}

for name in _all:
    _astr = r"""%s\s*[(]"""%name
    func_re[name] = re.compile(_astr)


import string
disallowed = '_' + string.uppercase + string.lowercase + string.digits

def _add_axis(fstr, name, repl):
    alter = 0
    if name in _args3:
        allowed_comma = 1
    else:
        allowed_comma = 0
    newcode = ""
    last = 0
    for obj in func_re[name].finditer(fstr):
        nochange = 0
        start, end = obj.span()
        if fstr[start-1] in disallowed:
            continue
        if fstr[start-1] == '.' \
           and fstr[start-6:start-1] != 'numpy' \
           and fstr[start-2:start-1] != 'N' \
           and fstr[start-9:start-1] != 'numarray' \
           and fstr[start-8:start-1] != 'numerix' \
           and fstr[start-8:start-1] != 'Numeric':
            continue
        if fstr[start-1] in ['\t', ' ']:
            k = start-2
            while fstr[k] in ['\t', ' ']:
                k -= 1
            if fstr[k-2:k+1] == 'def' or \
               fstr[k-4:k+1] == 'class':
                continue
        k = end
        stack = 1
        ncommas = 0
        N = len(fstr)
        while stack:
            if k>=N:
                nochange =1
                break
            if fstr[k] == ')':
                stack -= 1
            elif fstr[k] == '(':
                stack += 1
            elif stack == 1 and fstr[k] == ',':
                ncommas += 1
                if ncommas > allowed_comma:
                    nochange = 1
                    break
            k += 1
        if nochange:
            continue
        alter += 1
        newcode = "%s%s,%s)" % (newcode, fstr[last:k-1], repl)
        last = k
    if not alter:
        newcode = fstr
    else:
        newcode = "%s%s" % (newcode, fstr[last:])
    return newcode, alter

def _import_change(fstr, names):
    # Four possibilities
    #  1.) import numpy with subsequent use of numpy.<name>
    #        change this to import numpy.oldnumeric as numpy
    #  2.) import numpy as XXXX with subsequent use of
    #        XXXX.<name> ==> import numpy.oldnumeric as XXXX
    #  3.) from numpy import *
    #        with subsequent use of one of the names
    #  4.) from numpy import ..., <name>, ... (could span multiple
    #        lines.  ==> remove all names from list and
    #        add from numpy.oldnumeric import <name>

    num = 0
    # case 1
    importstr = "import numpy"
    ind = fstr.find(importstr)
    if (ind > 0):
        found = 0
        for name in names:
            ind2 = fstr.find("numpy.%s" % name, ind)
            if (ind2 > 0):
                found = 1
                break
        if found:
            fstr = "%s%s%s" % (fstr[:ind], "import numpy.oldnumeric as numpy",
                               fstr[ind+len(importstr):])
            num += 1

    # case 2
    importre = re.compile("""import numpy as ([A-Za-z0-9_]+)""")
    modules = importre.findall(fstr)
    if len(modules) > 0:
        for module in modules:
            found = 0
            for name in names:
                ind2 = fstr.find("%s.%s" % (module, name))
                if (ind2 > 0):
                    found = 1
                    break
            if found:
                importstr = "import numpy as %s" % module
                ind = fstr.find(importstr)
                fstr = "%s%s%s" % (fstr[:ind],
                                   "import numpy.oldnumeric as %s" % module,
                                   fstr[ind+len(importstr):])
                num += 1

    # case 3
    importstr = "from numpy import *"
    ind = fstr.find(importstr)
    if (ind > 0):
        found = 0
        for name in names:
            ind2 = fstr.find(name, ind)
            if (ind2 > 0) and fstr[ind2-1] not in disallowed:
                found = 1
                break
        if found:
            fstr = "%s%s%s" % (fstr[:ind],
                               "from numpy.oldnumeric import *",
                               fstr[ind+len(importstr):])
            num += 1

    # case 4
    ind = 0
    importstr = "from numpy import"
    N = len(importstr)
    while True:
        ind = fstr.find(importstr, ind)
        if (ind < 0):
            break
        ind += N
        ptr = ind+1
        stack = 1
        while stack:
            if fstr[ptr] == '\\':
                stack += 1
            elif fstr[ptr] == '\n':
                stack -= 1
            ptr += 1
        substr = fstr[ind:ptr]
        found = 0
        substr = substr.replace('\n', ' ')
        substr = substr.replace('\\', '')
        importnames = [x.strip() for x in substr.split(',')]
        # determine if any of names are in importnames
        addnames = []
        for name in names:
            if name in importnames:
                importnames.remove(name)
                addnames.append(name)
        if len(addnames) > 0:
            fstr = "%s%s\n%s\n%s" % \
                   (fstr[:ind],
                    "from numpy import %s" % \
                    ", ".join(importnames),
                    "from numpy.oldnumeric import %s" % \
                    ", ".join(addnames),
                    fstr[ptr:])
            num += 1

    return fstr, num

def add_axis(fstr, import_change=False):
    total = 0
    if not import_change:
        for name in _funcm1:
            fstr, num = _add_axis(fstr, name, 'axis=-1')
            total += num
        for name in _func0:
            fstr, num = _add_axis(fstr, name, 'axis=0')
            total += num
        return fstr, total
    else:
        fstr, num = _import_change(fstr, _funcm1+_func0)
        return fstr, num


def makenewfile(name, filestr):
    fid = file(name, 'w')
    fid.write(filestr)
    fid.close()

def getfile(name):
    fid = file(name)
    filestr = fid.read()
    fid.close()
    return filestr

def copyfile(name, fstr):
    base, ext = os.path.splitext(name)
    makenewfile(base+'.orig', fstr)
    return

def convertfile(filename, import_change=False):
    """Convert the filename given from using Numeric to using NumPy

    Copies the file to filename.orig and then over-writes the file
    with the updated code
    """
    filestr = getfile(filename)
    newstr, total = add_axis(filestr, import_change)
    if total > 0:
        print("Changing ", filename)
        copyfile(filename, filestr)
        makenewfile(filename, newstr)
        sys.stdout.flush()

def fromargs(args):
    filename = args[1]
    convertfile(filename)

def convertall(direc=os.path.curdir, import_change=False):
    """Convert all .py files in the directory given

    For each file, a backup of <usesnumeric>.py is made as
    <usesnumeric>.py.orig.  A new file named <usesnumeric>.py
    is then written with the updated code.
    """
    files = glob.glob(os.path.join(direc, '*.py'))
    for afile in files:
        convertfile(afile, import_change)

def _func(arg, dirname, fnames):
    convertall(dirname, import_change=arg)

def converttree(direc=os.path.curdir, import_change=False):
    """Convert all .py files in the tree given

    """
    os.path.walk(direc, _func, import_change)

if __name__ == '__main__':
    fromargs(sys.argv)
